/*----------------------------------------------------------------------------
 Copyright:      Wolfgang Scherr
 Author:         Wolfgang Scherr
 Remarks:        written from scratch, application specific project code
				 (it should NOT be necessary to touch any other code)
				 Irgendwann sollte ich mich selbst auf dt. oder en. Kommentare einigen :-)
 known Problems:  
 Version:        20.03.2009
 Description:    project specific implementations

 Dieses Programm ist freie Software. Sie können es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation veröffentlicht, 
 weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder späteren Version. 

 Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, 
 daß es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FÜR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdio.h>  // printf,...
#include <util/delay.h>  // _delay_xx
#include <string.h>  // memcpy, strlen,...

//Basis-Setup
#include "usart.h"
#include "lcd.h"
#include "glcd.h"
#include "ps2_kb.h"
#include "analog.h"
#include "timer.h"
#include "flashfs.h"

//Ethernet stack/driver
#include "stack.h"
#include "drivers/enc28j60.h"
#include "drivers/rtl8019.h"

//Webclients
#include "ntpc.h"
#include "dnsc.h"
#include "wol.h"

//Dämonen (Server)
#include "telnetd.h"

// Nachdem das System soweit über config.h vorkonfiguriert ist, kann man
// hier noch den eigenen Anwendungscode einfügen.
// Das konditionelle Kompilieren nach aktivierten Features kann man sich
// hier eventuell auch ersparen, macht den Code lesbarer.

// CALLBACK-FUNKTIONEN FÜR HTTP SERVER
// ====================================================================
#if USE_HTTPD
///Liefere einen Inhaltstyp für den Dateinamen oder 0, wenn nichts gefunden.
char my_httpd_filetype(char *f,char *t) {
	//We remove possible hierarchy separators.
	while (f[0]=='/') strcpy(f,&f[1]);

	printf_P(PSTR("\nType of '%s'\n"),f);
	// Process file here!
	// ------------------------------------------
#if USE_FLASHFS
	if (ffs_exists(f)) {
		//Eigentlich sollte man sich die Extension anschauen... :-)
		strcpy_P(t,PSTR("text/html"));
		return 1;
	}
#endif
	// ------------------------------------------

	//Default behavior...
	if (f[0]==0) {
		printf_P(PSTR("--> Asks for Index Page!\n"));
		strcpy_P(t,PSTR("text/html"));
		return 1;
	}

	// Not found!
	return 0;
}

///Liefere ein Datensegment (und die Länge) für den Dateinamen oder 0, wenn nichts gefunden.
unsigned int my_httpd_fileread(char *b,char *f,unsigned int start,unsigned int maxlen) {
	char f_buf[MAX_FILENAME_SIZE];

	//We remove possible hierarchy separators.
	while (f[0]=='/') strcpy(f,&f[1]);

	strncpy(f_buf,f,MAX_FILENAME_SIZE);

	// Process file here!
#if USE_FLASHFS
	if (f_buf[0]==0) strcpy_P(f_buf,PSTR("index.html"));
	printf_P(PSTR("Read '%s' from %d (%d)\n"),f_buf,start,maxlen);
	return ffs_read(f_buf,b,start,maxlen);
#else
	//Default behavior...
	printf_P(PSTR("Read '%s' from %d (%d)\n"),f_buf,start,maxlen);
	if (f_buf[0]==0) {
		strncpy_P(b,My_Page_IDX,maxlen);
		return sizeof(My_Page_IDX)-1;
	}
	return 0;
#endif
}

///Evaluiere und exekutiere Inhalt von POST-Inhalten.
char my_httpd_execute(char incomplete, char *n, char *v) {
	// Check end marker!
	if (strncmp_P(n,PSTR(POST_ENDMARKER),3)) return 1;
	// It is not the end marker, there must be more!
    if (incomplete) return 0;
	
	// Process data (name=n, value=n) here!
	printf_P(PSTR("Execute '%s'='%s'\n"),n,v);

	return 0;
}
#endif //USE_HTTPD
// ====================================================================

// ====================================================================
///Minimalseite für unser Flash-Filesystem 
///(sollte durch eine "richtige" Seite ersetzt und aus config.h
/// ausgelagert werden.)
PROGMEM char My_Page_IDX[] = {
	"<html><title>AVR Mini-Webserver</title>\n"
	"<body>This is the webpage defined in config.c.\n"
	"<form method=\"post\" action=\"/\">\n"
	"<input type=\"submit\" name=\""POST_ENDMARKER"\" value=\"MY TEST\">\n"
	"</body></html>"};
// ====================================================================


/**Hook1: Sehr frühe Initialisierung, gleich nach Reset. 
   Hier kann man z.B. Parameter aus dem EEPROM lesen, wie:
   eigene IP Adresse, Netzmaske, Router-IP (Gateway), DNS-Server-IP, u.v.m.
   und die Standardstreams (!) festlegen.
*/
void pre_HW_init(void) {
	#if USE_GLCD_STREAM
		// assign graphics LCD as standard out stream for printf
		stdout = &glcd_stream;	
	#elif USE_LCD_STREAM
		// assign standard LCD as standard out stream for printf
		stdout = &lcd_stream;	
	#elif USE_USART_STREAM
		// otherwise assign USART as standard out stream for printf
		stdout = &usart_stream;
	#else
		#error "Can not assign stdout stream to an output device!"
	#endif
}

// ====================================================================

/**Hook2: Nach Initialisierung der Basisfunktionen.
   (z.B. Keyboard, UART, LCD, ... inclusive Ethernet-Stack.) 
   Hier kann man auch Serveraddressen für Web-Clients wie NTP vor deren 
   Start noch per DNS nachfragen.
*/
void pre_APP_init(void) {
	printf_P(PSTR("\n\nSystem Starting...\n"));
	printf_P(PSTR("\nBuild: "__DATE__", "__TIME__"\n"));
	printf_P(PSTR("GCC Version: "__VERSION__"\n\n"));

	printf_P(PSTR("MAC (%02x:%02x:%02x:%02x:%02x:%02x)\n"),
				mymac[0],mymac[1],mymac[2],mymac[3],mymac[4],mymac[5]);
	printf_P(PSTR("IP   %1i.%1i.%1i.%1i\n"), 
				myip[0]     , myip[1]     , myip[2]     , myip[3]);
	printf_P(PSTR("MASK %1i.%1i.%1i.%1i\n"), 
				netmask[0]  , netmask[1]  , netmask[2]  , netmask[3]);
	printf_P(PSTR("GW   %1i.%1i.%1i.%1i\n"), 
				router_ip[0], router_ip[1], router_ip[2], router_ip[3]);
	#if USE_DNS
		printf_P(PSTR("DNS  %1i.%1i.%1i.%1i\n"), 
				dns_server_ip[0], dns_server_ip[1], dns_server_ip[2], dns_server_ip[3]);
	#endif //USE_DNS

	#if USE_NTP	
        #if USE_DNS
			//Wir möchten eine aktuelle IP für den NTP Server verwenden,
			//also schauen wir mal nach...
			printf_P(PSTR("3.de.pool.ntp.org: "));
			if ( dns_resolve("3.de.pool.ntp.org") == 0)
			{
				//Wir übernehmen die neue IP.
				(*(unsigned long*)&ntp_server_ip[0]) = (*(unsigned long*)&dns_resolved_ip[0]);
				printf_P(PSTR("ok\n"));
			}
			else
			{
				//Ging daneben, wir hoffen die gespeicherte IP passt!
				printf_P(PSTR("error\n"));
				_delay_ms(2000);
			}
        #endif //USE_DNS
		
		//Wir wollen NTP für die Systemzeit verwenden, zeigen wir die Server-IP.
		printf_P(PSTR("NTP  %1i.%1i.%1i.%1i\n"), 
				ntp_server_ip[0], ntp_server_ip[1], ntp_server_ip[2], ntp_server_ip[3]);
	#endif //USE_NTP

	#if USE_WOL
		//Netzwerkclient mit gegebener MAC einschalten, wir brauchen
		//ihn z.B. für spätere Kommunikation. (nur als Beispiel)
		unsigned char wol_mac[]={0x00,0x11,0x14,0x22,0x09,0x18};
		printf_P(PSTR("\nWOL: %02x:%02x:%02x:%02x:%02x:%02x\n"),
				wol_mac[0],wol_mac[1],wol_mac[2],wol_mac[3],wol_mac[4],wol_mac[5]);
		wol_request(wol_mac);
		_delay_ms(2000);
	#endif

	//Overwrite HOOKS for HTTP-Daemon
	#if USE_HTTPD
		printf_P(PSTR("Assigning HTTP handler.\n"));
		//Requests a file type for a given filename
		file_type_p = &my_httpd_filetype;
		//Executes a given variable with given content
		exec_var_p = &my_httpd_execute;
		//Reads a data segment from a given filename
		file_read_p = &my_httpd_fileread;
	#endif

	//Filesystem "laden"
	#if USE_FLASHFS
		ffs_add(PSTR("index.html"),My_Page_IDX,sizeof(My_Page_IDX));
	#endif
}

/**Hook3: Highlevel Applikationsinitialisierung.
   Hier ist das System prinzipiell betriebsbereit. Man sollte checken, 
   ob alle Basisfunktionen ok sind und dann die eigene Applikation 
   (soferne nötig) noch nach-initialisieren. 
*/
void post_init(void) {

	#if USE_NTP	
		if ( ntp_ok )
		{
			//Alles klar, wir schreiben die Zeit auf die ser. Schnittstelle.
			ntp_struct dt;
			decode_time(get_time(), &dt);
			printf_P(PSTR("%s, %s\n"), dt.datestr, dt.timestr);
		}
		else
		{
			//Schiefgegangen, Fehler anzeigen. Systemzeit ist nicht aktuell.
			printf_P(PSTR("NTP Err.\n"));
			_delay_ms(2000);
		}
	#endif //USE_NTP

	_delay_ms(3000);
	printf_P(PSTR("\nMenu:\n"));
	#if USE_ADC
		printf_P(PSTR("a...show ADC values\n"));
	#endif
	#if USE_GLCD
		printf_P(PSTR("g...graphic LCD test\n"));
		printf_P(PSTR("m...graphic LCD motion\n"));
	#endif
	#if USE_NTP
		printf_P(PSTR("t...show time\n"));
	#endif
	#if USE_USART_STREAM || USE_GLCD_STREAM || USE_TELNET_STREAM || USE_LCD_STREAM
		printf_P(PSTR("s...stream printf test\n"));
	#endif
	/*
	#if GET_WEATHER
		printf_P(PSTR("w...read weather report from web\n"));
	#endif
	
	#if USE_MAIL
		printf_P(PSTR("r...send email report\n"));
	#endif
	*/
	printf_P(PSTR("\nSystem READY.\n"));
}

//Globals are not very efficient, but as it is only for testing...
char motion = 0;
char x=10;
char y=10;
char dx=5;
char dy=5;

/**MAIN HOOK: Diese Routine wird aus der Hauptschleife heraus regelmäßig aufgerufen. 
   Hier muss man auch auf auf etwaige System-Events (z.B. USART) reagieren.
   Generell programmiert man hier die eigene Systemapplikation.
*/
void do_main_task(void) {
		char c = 0;

		#if USE_GLCD
			if (motion) {
				glcd_clrgraphics();
				glcd_circle(x,y,9);
				x+=dx;
				y+=dy;
				if ((y>GLCD_Y_RES-15)||(y<15)) dy=-dy;
				if ((x>GLCD_X_RES-15)||(x<15)) dx=-dx;
				_delay_ms(100);
			}
		#endif
		
		#if USE_PS2KB_STREAM
			//Keyboard Zeichen holen (PS/2 keyboard ist zeichenbasiert)
			if (kb_ischar()) {
				c=fgetc(&kb_stream);
			}
		#endif

		#if USE_USART_STREAM
			//USART Zeichen holen (usart i/o ist zeichenbasiert)
			if (usart_ischar()){
				c = fgetc(&usart_stream);
			}
		#endif //USE_USART_STREAM

		#if USE_TELNET
			//TELNET Zeichen holen (telnet i/o ist zeilenbasiert)
			if (telnet_ischar()){
				c = fgetc(&telnet_stream);
				//We ignore low-level TELNET-commands.
				if (c==IAC) {
					fgetc(&telnet_stream);
					fgetc(&telnet_stream);
					c=0;
				}
				//We ignore '\n', as we add it later after '\r'
				if (c==10) c=0;
			}
		#endif //USE_TELNET

		if (c==13) putchar('\n');
		if (c) putchar(c);
		
		#if USE_GLCD
			if (c=='m') { 
				if (motion) 
					motion=0; 
				else 
					motion=1;
			}
			if (c=='g') {
				motion=0; 
				#if USE_GLCD_STREAM
					glcd_clrgraphics();
					glcd_clrtext();
					// write characters
					fprintf_P(&glcd_stream,PSTR("HOME"));
					glcd_pos(0,1);
					fprintf_P(&glcd_stream,PSTR("Scroll out..."));
					// write characters (first line shows also 0x0 as "illegal char.")
					glcd_pos(0,2);
					putc(0x0,&glcd_stream);
					fprintf_P(&glcd_stream,PSTR("1234567890!'$%%&/()=?\"\\@+*#_|\n"));
					fprintf_P(&glcd_stream,PSTR("ABCDEFGHIJKLMNOPQRSTUVWXYZ;,\n"));
					fprintf_P(&glcd_stream,PSTR("abcdefghijklmnopqrstuvwxyz:."));

					glcd_pos(28,7);
					fprintf_P(&glcd_stream,PSTR("WRAPAROUND"));
					fprintf_P(&glcd_stream,PSTR("-Stream test.\n"));
				#else
					glcd_pos(0,0);
					glcd_write_str("GLCD Test",9);
				#endif //USE_GLCD_STREAM

				// Graphics test
				for(int i=0;i<64;i++)
					glcd_setpixel(i,i);
				for(int i=0;i<64;i++)
					glcd_setpixel(63-i,i+64);
				glcd_line(0,0,239,127);
				glcd_rect(10,10,230,118);
				glcd_circle(120,64,50);

				#if USE_GLCD_STREAM
					_delay_ms(1000);
					// Scroll test
					glcd_pos(0,16);
					fprintf_P(&glcd_stream,PSTR("SCROLLING."));
					_delay_ms(1000);
					// Scroll test
					fprintf_P(&glcd_stream,PSTR("\n"));
				#endif
			}
		#endif //USE_GLCD

		#if USE_ADC
			if (c=='a') {
				printf_P(PSTR("\nADC"));
				// Im Display zeigen wir die ADC Werte (hex)
				for(char a=0;a<8;a++)
					printf_P(PSTR("%2i:%03x"),a,adc((int)a));
				printf_P(PSTR("\n"));
			}
		#endif //USE_ADC

		#if USE_NTP
			if (c=='t') {
				//Get time in readable form
				ntp_struct dt;
				decode_time(get_time(), &dt);
				printf_P(PSTR("\nTime:%s\n"),dt.timestr);
			}
		#endif //USE_NTP
		
		#if USE_USART_STREAM
			if (c=='s') {
				fprintf_P(&usart_stream,PSTR("-STREAM TEST\n"));
			}
		#endif

		#if USE_GLCD_STREAM
			if (c=='s') {
				fprintf_P(&glcd_stream,PSTR("-STREAM TEST\n"));
			}
		#endif

		#if USE_LCD_STREAM
			if (c=='s') {
				fprintf_P(&lcd_stream,PSTR("-STREAM TEST\n"));
			}
		#endif

		#if USE_TELNET_STREAM
			if (c=='s') {
				fprintf_P(&telnet_stream,PSTR("-STREAM TEST\n"));
			}
		#endif

		// Ping checken (sofern interessant) und flag ggf. wieder löschen
		#if ENABLE_PING || ENABLE_BROADCAST_PING
			if(ping.sent)
			{
				printf_P(PSTR("\nSent PING to %i.%i.%i.%i\n"),
				       ping.ip1[0],ping.ip1[1],ping.ip1[2],ping.ip1[3]); 
				ping.sent = 0;
			}
			if(ping.result)
			{
				printf_P(PSTR("\nGet PONG: %i.%i.%i.%i\n"),
					    ping.ip1[0],ping.ip1[1],ping.ip1[2],ping.ip1[3]); 
				ping.result = 0;
			}
		#endif //*PING

		/*		
        //Wetterdaten empfangen (Testphase)
        #if GET_WEATHER
			if (c=='w') {
				http_request ();
			}
        #endif
        
        //Versand von E-Mails
        #if USE_MAIL
			if ((mail_enable == 1) && (c=='r'))
			{
				mail_enable = 0;
				mail_send();
			}
        #endif //USE_MAIL
		*/

}
