/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich, reworked by Wolfgang Scherr
 Remarks:        Rework: config.h, unschoene Codeabhngigkeiten entfernt,
				 vollstndige Trennung Basis-System/Applikationscode.
 known Problems: none
 Version:        12.02.2009
 Description:    Webserver Config-File

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#ifndef _CONFIG_H_
	#define _CONFIG_H_	

	// MCU Konfiguration
	// =================

		//Taktfrequenz
		//#define F_CPU 20000000UL
		#define F_CPU 16000000UL	
		//#define F_CPU 14745600UL
		//#define F_CPU 11059200UL
			
		//Konfiguration der PORTS (HEX)
		// OUTx: 1=OUTPUT / 0=INPUT
		// SETx: 1=HIGH/Pull-Up / 0=LOW/Off
								 // AVR-NET-IO Belegung:
		#define OUTA 		0x00 // PORTA0..3 are inputs on J3, PORTA4...7 are on J7/J9
		#define SETA 		0xFF // pull-up enabled 
		#define OUTB 		0x00 // Achtung!!!! an PORTB ist der ENC, 
		#define SETB 		0x00 // nur ndern wenn man wei, was man macht!
		#define OUTC 		0xFF // outputs on J3 (Centronics-Port)
		#define SETC 		0x00 // no pullup
		#define OUTD 		0x00 // EXT Port und USART (PB0/PB3 ist auch EXT) 
		#define SETD 		0xFF // pull-up enabled 	

	// Timer Konfiguration
	// ===================

		//Systemtimer verwenden (1s Takt - braucht fast jedes Modul...)
		#define USE_TIMER 1

		//Timertakt intern oder extern
		#define EXTCLOCK 0 //0=Intern 1=Externer Uhrenquarz
		
		//Anzahl and mglichen Flag & Watchdog-Eintrgen 
		//(z.B. Netzwerk-Stack braucht 2, NTP braucht 2, 
		//      DNS braucht 1, DHCP braucht 2,...)
		#define MAX_TIMER_ENTRIES 10
		
		//Hardware Reset - wird vom Timer-Watchdog aufgerufen.
		//Man knnte hier ggf. noch einen Port toggeln, um diesen Reset
		//extern anzuzeigen.
		#define RESET() {cli(); asm("ldi r30,0"); asm("ldi r31,0"); asm("ijmp");}

	// USART Konfiguration
	// ===================
	
		#define USE_USART 1  // 0 setzen, wenn CAM aktiviert werden soll!

		//Stream definieren?
		#define USE_USART_STREAM USE_USART  // USE_USART oder 0

		//Baudrate der seriellen Schnittstelle
		#define BAUDRATE 9600
		
		//Diverse Settings wie Puffergre und lokales Echo
		#define USART_ECHO	0
		#define USART_BUFFER_SIZE	50

	// ADC Konfiguration
	// =================

		//AD-Wandler benutzen?
		#define USE_ADC 1

		//interne Referenzquelle gewhlt (bei 0 wird ext. Quelle angeschlossen)
		#define INT_REF 1

	// PS2/AT-Tastatur Konfiguration
	// =============================

		//PS2-Tastatur aktivieren
		#define USE_PS2KB 0

		//Stream definieren?
		#define USE_PS2KB_STREAM USE_PS2KB   // USE_PS2KB oder 0

		//Anschlussdefintion
			// CLOCKPIN (muss INTx-Pin sein !!!)
			#define PS2KB_CLKDDR DDRD  // DDR Register
			#define PS2KB_CLKPIN PIND	// PIN Register
			#define PS2KB_CLK PD2		// Pinbezeichnung
			//Interruptquelle (muss zu Anschluss des PS2-Clockpin passen!)
			#define PS2KB_CLK_INT 0		//PD2==INT0
			// DATAPIN
			#define PS2KB_DATADDR DDRD  // DDR Register
			#define PS2KB_DATAPIN PIND	// PIN Register
			#define PS2KB_DATA PD3		// Pinbezeichnung

		//Sprachwahl/Tastaturlayout
		#define PS2KB_GERMAN  1
		#define PS2KB_ENGLISH 0

	// LCD Konfiguration
	// =================

		//LCD Routinen mit einbinden (HD44780 kompatibel)
		#define USE_LCD 0
		
		//Stream definieren?
		#define USE_LCD_STREAM USE_LCD  // USE_LCD oder 0

		//LCD Status soll rckgelesen werden (nur in parallelen Modi)
		//Kostet etwas Code, macht den Zugriff aber schneller.
		#define LCD_PAR_READ 1
		
		//LCD Gre (1*16, 2*16, 1*20, 2*20, 4*16, 4*20 erlaubt)
		//#define LCD_SIZE 4*20
		#define LCD_SIZE 4*20

		//Backlight-Kontrolle einbinden
		#define USE_LCD_BACKLIGHT 1
			//Externe Backlightschaltung invertiert das Signal nicht
			#define LCD_BL_NOTINV 0

		//LCD Anschlussvarianten (nur eine davon aktivieren!!!)

		//Zuerst ein paar vordefinierte Flle
		
			//LCD am AVR-NET-IO EXT PORT 
			// --> siehe orig. Datenblatt AVR-NET-I/O
			#define LCD_EXT_PORT 1
				//Belegung ist hier fix, "allgemeinere" Ansteuerungen unten: 
				//	Pin1	(PD2): R/W (LCD Pin 5)
				//	Pin2	(PD3): RS (LCD Pin 4)
				//	Pin3-6(PD4-7): DB4-7 (LCD Pin 11-14)
				//	Pin7	(PB0): E (LCD Pin 6)
				//	Pin8	(PB3): (offen bzw. Backlight Pin)
				//	Pin9	(GND): Vss (LCD Pin 1)
				//	Pin10	(+5V): Vdd (LCD Pin 2) 
			
			//LCD seriell angesteuert (3 Leitungen), Schaltung/Info unter: 
			//  	http://www.ulrichradig.de/home/index.php/avr/lc-display
			//		entspricht den Settings im Sourcecode
			#define LCD_SerLCD 0
				//Belegung ist hier fix, "allgemeinere" Ansteuerungen unten: 
				//  PC3: Data into shift register - A,B on 74xx164
				//  PC4: Clock into shift register - CLK on 74xx164
				//  PC2: E (LCD Pin 6)
				//
				//       QA on 74xx164:    RS (LCD Pin 4)
				//       QD-QG on 74xx164: DB4-DB7 (LCD Pin 11-14)
				//       QH on 74xx164:    Backlight

		// Hier die generische Installation

			// LCD seriell angesteuert
			#define LCD_SER 0
				//Belegung:
				//E (LCD Pin 6) - DATA_ENABLE
				#define PORT_LCD_E  PORTC
				#define DDR_LCD_E   DDRC
				#define LCD_E       2
				//Anschlsse am Schieberegister
				#define PORT_LCD_DATA        PORTC
				#define DDR_LCD_DATA         DDRC
				#define LCD_DATA             4
				#define PORT_LCD_CLOCK       PORTC
				#define DDR_LCD_CLOCK        DDRC
				#define LCD_CLOCK            3
				//Pins "hinter" Schieberegister
				//RS (LCD Pin 4)
				#define LCD_RS_PIN       0
				//Backlight (LCD Pin 15/16 ber ext. Transistor)
				#define LCD_BL_PIN       7
				//DB4-7 (LCD Pin 11-14)
				#define LCD_DATA_MSB_PIN 6

			//LCD 4bit-parallel angesteuert
			#define LCD_PAR 0
				//Belegung:
				//DB4-7 (LCD Pin 11-14) - IN/OUT/DDR mssen gleich sein!!!
				#define PORT_LCD_DATA_OUT    PORTC
				#define PORT_LCD_DATA_IN     PINC
				#define DDR_LCD_DATA         DDRC
				//vom ausgewhlten Port kann man das High-Nibble (7-4) oder
				//das Low-Nibble (3-0) verwenden (andere Konfig. nicht mglich)
				#define LCD_DATA_LOW_NIBBLE 0  //1=Low Nibble, 0=High Nibble
				//E (LCD Pin 6)
				#define PORT_LCD_E  PORTC
				#define DDR_LCD_E   DDRC
				#define LCD_E       2
				//R/W (LCD Pin 5)
				#define PORT_LCD_RW PORTC
				#define DDR_LCD_RW  DDRC
				#define LCD_RW      1
				//RS (LCD Pin 4)
				#define PORT_LCD_RS PORTC
				#define DDR_LCD_RS  DDRC
				#define LCD_RS      0
				//Backlight (LCD Pin 15/16 ber ext. Transistor)
				#define PORT_LCD_BL PORTC
				#define DDR_LCD_BL  DDRC
				#define LCD_BL      3

	// Graphics-LCD Konfiguration
	// ==========================

		//Graphidisplay aktivieren
		#define USE_GLCD 0

		//Stream definieren?
		#define USE_GLCD_STREAM USE_GLCD   // USE_GLCD oder 0

		//Auflsung des Displays in Pixel
		//Achtung, ev. muss man den Code auch noch anpassen...
		#define GLCD_X_RES 240
		#define GLCD_Y_RES 128

		//8bit Datenbus
			#define DDR_GLCD_DB DDRC
			#define PORT_GLCD_DB PORTC
			#define PIN_GLCD_DB PINC
		//Kontrollsignale
			#define DDR_GLCD_WR_OUT DDRA
			#define PORT_GLCD_WR_OUT PORTA
			#define PORT_GLCD_WR PA2
			#define DDR_GLCD_RD_OUT DDRA
			#define PORT_GLCD_RD_OUT PORTA
			#define PORT_GLCD_RD PA3
			#define DDR_GLCD_CD_OUT DDRA
			#define PORT_GLCD_CD_OUT PORTA
			#define PORT_GLCD_CD PA1
			//Braucht man u.U. nicht, ev. Code in Zukunft verbessern um
			//diesen Pin eliminieren zu knnen.
			#define DDR_GLCD_CE_OUT DDRA
			#define PORT_GLCD_CE_OUT PORTA
			#define PORT_GLCD_CE PA0

	// KAMERA Konfiguration
	// ====================

	//---- mangels CAM/SERVO nicht getestet und nicht eingebunden

	// NETZWERK Konfiguration
	// ======================

		//Ethernet Basispaket aktivieren (USE_TIMER muss dann 1 sein)
		#define USE_ETHERNET 1
		
		//MAC Adresse des Ethernet-Interfaces (initial, anpassen!!!)
		#define MYMAC 		{0x00,0x22,0xf9,0x01,0x02,0x03}
		
		//Eigene IP/Netzwerkmaske und IP des Routers (initial, anpassen!!!)
		#define MYIP		IP(192,168,1,99)
		#define NETMASK		IP(255,255,255,0)
		#define ROUTER_IP	IP(192,168,1,253)
		
		//Enable direct/broadcasted ping reply
		#define ENABLE_PING 1
		#define ENABLE_BROADCAST_PING 1

		//Watchdog timer for the ENC2860, resets the stack if timeout occurs
		#define WTT 1200 //Watchdog timer count in seconds

		//ETH_M32_EX (www.ulrichradig.de), AVR-NET-I/O (pollin.de)
		#define USE_ENC28J60 1

			//Dieses Pin wird direkt angesprochen, kann verschoben werden.
			//(3=Ulrich's Webserver, 4=AVR-NET-I/O, andere nicht bekannt)
			#define ENC28J60_PIN_CS		4
			//Diese Pins mssen dem SPI Peripheral entsprechen
			//sonst geht nichts!
			#define ENC28J60_PIN_SCK	7
			#define ENC28J60_PIN_MISO	6
			#define ENC28J60_PIN_MOSI	5

			//Zugehriger Port (A...D) & Pinnummern (0...7)
			//Port muss dem SPI Peripheral entsprechen, sonst geht nichts!
			#define ENC28J60_DDR		DDRB
			#define ENC28J60_PORT		PORTB

		//Holger Buss (www.mikrocontroller.com) Mega32-Board
		#define USE_RTL8019 0
		
			//---- sw: RTL8019 mangels Testboard nicht berprft -
			//Implementierung "as is" bernommen und MAC setup angepasst.

	// NETZWERK Applikationen (Clients/Daemons)
	// ========================================

	// ------------------------------------------

		//DHCP-Lookup verwenden (nicht gestestet/eingebunden)
		#define USE_DHCP   1

	// ------------------------------------------

		//DNS-Server
		#define USE_DNS 1 //1 = DNS Client on
		//IP des DNS-Servers (blicherweise gleich wie Router)
		#define DNS_IP  ROUTER_IP

		//Timeout fr DNS request
		#define DNS_TIMEOUT 30
		
	// ------------------------------------------
	
		//NTP-Server
		#define USE_NTP		1 //1 = NTP Client on
		//IP des NTP-Servers z.B. Server 1.de.pool.ntp.org
		//Wenn man DNS hat, kann man im config.c einen NS-lookup vor dem Init durchfhren
		#define NTP_IP		IP(77,37,6,59)

		//Timeout fr NTP request
		#define NTP_TIMEOUT 5
		
		//NTP based timer refresh time
		#define NTP_REFRESH 1140

	// ------------------------------------------

		//Wake-On-Lan-Applikation aktivieren
		#define USE_WOL	1 //1 = WOL on

	// ------------------------------------------

		//Telnet Daemon aktivieren
		#define USE_TELNET 0

		//Stream definieren?
		#define USE_TELNET_STREAM USE_TELNET  // USE_TELNET oder 0

		//Buffer wie USART
		#define TELNET_BUFFER_SIZE USART_BUFFER_SIZE
 
 		//TELNET-Serverport
		#define TELNET_APP_PORT 23

	// ------------------------------------------

		//HTTP Daemon aktivieren
		#define USE_HTTPD 1

		//Webserver mit Passwort? (0 == mit Passwort)
		#define HTTP_AUTH_DEFAULT	0
		
		//AUTH String "USERNAME:PASSWORT" max 14Zeichen 
		//fr Username:Passwort
		#define HTTP_AUTH_STRING "admin:avr-net-io"
		
		//Ende-Marker fr Formulare und entsprechenden POST-Paketen
		#define POST_ENDMARKER "SUBMIT"
		
		//HTTP-Serverport
		#define HTTPD_PORT 80

		//max. Lnge von Filenamen
		#define MAX_FILENAME_SIZE 13

		//max. Lnge von Inhaltstypen
		#define MAX_FILETYPE_SIZE 13

		//max. Lnge von Variablennamen
		#define MAX_NAME_SIZE 8

		//max. Lnge von Variableninhalten
		#define MAX_VAL_SIZE 8

	// ------------------------------------------

		//Email vesand benutzen? Konfiguration des
		//Emailclient in der Sendmail.h (nicht gestestet/eingebunden)
		#define USE_MAIL        0
		
	// ------------------------------------------

		//Empfang von Wetterdaten auf der Console (ber HTTP_GET)
		// (nicht gestestet/eingebunden)
		#define GET_WEATHER     0
    
	// Filesysteme
	// ========================================

	// ------------------------------------------

		//Einfaches Flash-Filesystem
		#define USE_FLASHFS 1
	
		//Anzahl Eintrge
		#define FFS_MAX_ENTRIES 20

	// ------------------------------------------

		//FAT16 ber SD-Karte (nicht implementiert)
		#define USE_SDFAT16 0
		
	//===============================================================

		//Prototypen fr eigenen Anwendungscode.
		//Der erste Teil wird vom System erwartet, nicht verndern!:
		void pre_HW_init(void);
		void pre_APP_init(void);
		void post_init(void);
		void do_main_task(void);

		char(*file_type_p)(char *, char *);
		char(*exec_var_p)(char, char *,char *);
		unsigned int (*file_read_p)(char *buf,char *filename,unsigned int start,unsigned int maxlen);

		//Hier kann man noch eigene Prototypen anlegen, die "Systemweit"
		//sichtbar sind - soferne ntig...

	//===============================================================

		//ACHTUNG: Debugging kostet u.U. mehr RAM!!! 
		
		//Wenn man Fehler sucht, muss man aufpassen, dass man nicht 
		//"Geister" aufgrund von Speichermangel jagt!!!
		//
		//  Wenn man kein gutes Debugging-Environment hat, so vorgehen:
		//	 Zuerst: AVR035: lesen und beherzigen (leider ist der Code
		//           noch nicht so optimal,wie er sein sollte).
		//   1) Nicht bentigte Funktionen bis auf die (vermeindlich)
		//      Fehlerhafte abschalten!!! (--> USE_xxxx 0)
		//   2) RAM-Bedarf (temporr) reduzieren, z.B. in stack.h Anzahl 
		//      der mglichen Eintrge reduzieren (MAX_xxx_ENTRY) 
		//	 3) Kompilieren und berprfen, ob der Fehler nach wie vor
		//		existiert - nur dann kann man sich halbwegs sicher sein,
		//      dass man keinem Stackberlauf zum Opfer gefallen ist.
		//   4) Erst jetzt die Debugfunktion aktivieren, aber die
		//      "Sparmanahmen" aktiv lassen! Die meisten Debug-Outputs
		//      brauchen (leider noch) recht viel Speicher...
		//   5) Immer auf den Speicherbedarf achten! (Mapfiles,...)
		
		//Ethernet-Debugging ber printf (stdout muss umgeleitet werden!)
		#define STACK_DEBUG 0
	  	//DHCP-Debugging ber printf (stdout muss umgeleitet werden!)
		#define DHCP_DEBUG 0
	  	//DNS-Debugging ber printf (stdout muss umgeleitet werden!)
		#define DNS_DEBUG 0
		//NTP-Debugging ber printf (stdout muss umgeleitet werden!)
		#define NTP_DEBUG 0
		//WOL-Debugging ber printf (stdout muss umgeleitet werden!)
		#define WOL_DEBUG 0
		//Telnet-Debugging ber printf (stdout muss umgeleitet werden!)
		#define TELNET_DEBUG 0
		//HTTP-Debugging ber printf (stdout muss umgeleitet werden!)
		#define HTTP_DEBUG 0
		//FLASHFS-Debugging ber printf_P (stdout muss umgeleitet werden!)
		#define FLASHFS_DEBUG 0
		//MAIN-Debugging ber printf_P (stdout muss umgeleitet werden!)
		//Relativ "save"...
		#define MAIN_DEBUG 0

#endif //_CONFIG_H
