/*----------------------------------------------------------------------------
 Copyright:      Wolfgang Scherr
 Author:         Wolfgang Scherr
 Remarks:        from scratch written, influenced by several web sources,
				 especially: webpage.h from Ulrich Radigs Webserver. 
				 Irgendwann sollte ich mich selbst auf dt. oder en. Kommentare einigen :-)
 known Problems: may cause garbled output for some keys, still "alpha" code 
 Version:        22.03.2009
 Description:    Simple (flash) FS support

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"

#if USE_FLASHFS

	#include <avr/pgmspace.h>
	#include <stdio.h>

	#if FLASHFS_DEBUG
		#warning "Debugging of FLASH FILESYSTEM"
		#define FFS_DEBUGP	printf_P 
	#else
		#define FFS_DEBUGP(...)	
	#endif

	#include "flashfs.h"

	typedef struct {
		PGM_P file_name;
		PGM_P file_content;
		unsigned int file_len;
	} FFS_ITEM;

	FFS_ITEM flashfs[FFS_MAX_ENTRIES];

	///Initialize flash file system (FFS)
	void ffs_init(void)
	{
		FFS_DEBUGP(PSTR("Init FFS\n"));
		for(unsigned int i=0;i<FFS_MAX_ENTRIES;i++)
			flashfs[i].file_name=NULL;
	}
	
	///Add an fileentry
	unsigned char ffs_add(PGM_P filename, PGM_P filecontent, unsigned int len) 
	{	unsigned int i=0;
	
		FFS_DEBUGP(PSTR("Add file with length %d\n"),len);

		while (flashfs[i].file_name) {
			if (++i==FFS_MAX_ENTRIES) return 1; // ERROR
		}
		
		flashfs[i].file_name = filename;
		flashfs[i].file_content = filecontent;
		flashfs[i].file_len = len;
		return 0;
	}
		
	///Checks if a file exists
	unsigned char ffs_exists(char *filename)
	{	unsigned int i=0;
	
		FFS_DEBUGP(PSTR("Check for file %s\n"),filename);

		while (flashfs[i].file_name) {
			if (!strcmp_P(filename,flashfs[i].file_name)) return 1; //FOUND
			if (++i==FFS_MAX_ENTRIES) return 0; // NOT FOUND
		}
		return 0; // NOT FOUND
	}
	
	///Reads a file into a buffer
	///with a certain filename from a position and a given length
	unsigned int ffs_read(char* filename, char *buf, unsigned int pos, unsigned int len)
	{   unsigned int remain, i=0;
	
		FFS_DEBUGP(PSTR("Read file %s\n"),filename);
		while (flashfs[i].file_name) {
			if (!strcmp_P(filename,flashfs[i].file_name)) break; //FOUND
			if (++i==FFS_MAX_ENTRIES) return 0; // NOT FOUND
		}

		if (pos>=flashfs[i].file_len) return 0; // NO MORE DATA
		remain = flashfs[i].file_len-pos;

		if (remain>len) remain=len;
		memcpy_P(buf,(flashfs[i].file_content)+pos,remain);

		FFS_DEBUGP(PSTR("Got %d bytes.\n"),remain);
		return remain;
	}
#endif //USE_FLASHFS
