/*----------------------------------------------------------------------------
 Copyright:      Wolfgang Scherr
 Author:         Wolfgang Scherr
 Remarks:        from scratch written, influenced by several web sources,
				 especially: T6963C Datasheet from TOSHIBA 
				 Irgendwann sollte ich mich selbst auf dt. oder en. Kommentare einigen :-)
 known Problems:  
 Version:        15.02.2009
 Description:    T6963C-type graphics display support

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"

#if USE_GLCD
	#include <avr/io.h>
	#include <stdlib.h>
	#include <util/delay.h>
	
#if !USE_GLCD_STREAM
	#include <avr/pgmspace.h>
	#include <stdarg.h>
	#include <ctype.h>
	#include <string.h>
#else
	#include <stdio.h>
#endif

	#include "glcd.h"


//----------------------------------------------------------------------------
//Init Display
void glcd_init() {
	PORTA |= (1<<PA2); 
	PORT_GLCD_WR_OUT |= (1<<PORT_GLCD_WR);
	PORT_GLCD_RD_OUT |= (1<<PORT_GLCD_RD);
	PORT_GLCD_CE_OUT |= (1<<PORT_GLCD_CE);
	DDR_GLCD_WR_OUT |= (1<<PORT_GLCD_WR);
	DDR_GLCD_RD_OUT |= (1<<PORT_GLCD_RD);
	DDR_GLCD_CE_OUT |= (1<<PORT_GLCD_CE);
	DDR_GLCD_CD_OUT |= (1<<PORT_GLCD_CD);
	DDR_GLCD_DB = 0xFF;
	// CE always active
	//PORT_GLCD_CE_OUT &= ~(1<<PORT_GLCD_CE);

	glcd_write_byte(DCMD,1); // DISPLAY OFF

	// TEXT DISPLAY RAM FROM 0x0000 to 0x01DF (240x128)
	glcd_write_cmd(TXHOME,0x0000);
	// GRAPHICS DISPLAY RAM FROM 0x0200 to 0x10FF (240x128)
	glcd_write_cmd(GRHOME,0x0200);
	// TEXT WIDTH IS 30 (240/8)
	glcd_write_cmd(TXAREA,GLCD_X_RES>>3);
	// GRAPHICS WIDTH IS 30 (240/8)
	glcd_write_cmd(GRAREA,GLCD_X_RES>>3);

	// CHARACTER GENERATOR RAM: OFFSET=3 --> 0x1800-0x1FFF
	glcd_write_cmd(OFFSET,0x0003);

	// CHARACTER GENERATOR RAM for all 256 characters
	// (0x7f and lower positions are not visible in CG ROM mode)
	// So we start at 0x80 POSITION (0x1800+8*0x80)
	glcd_write_cmd(ADPSET,0x1C00);
	// set up character graphics lines used as "nonprintable character"
	unsigned char buf[]={0x2A,0x15,0x2A,0x15,0x2A,0x15,0x2A,0x15};
	glcd_write_buf(buf,8);

	// Cursor mode
	glcd_write_byte(CUR1,1);	//1-line text cursor

	// Graphics mix mode
	glcd_write_byte(M_OR,1);

	// Display enable
	glcd_write_byte(DCMD|D_CB|DTEXT|DGRA,1); // Enable Cursor, Text ON, Graphics on
}

void glcd_write_cmd(unsigned char cmd, unsigned int val) {
	unsigned int timeout;
	//
	PORT_GLCD_CE_OUT &= ~(1<<PORT_GLCD_CE);
	//Wait until LCD is ready
	timeout=0; while (((glcd_read_byte(1)&0x03)!=0x03) && (timeout++ < 500)) {};
	PORT_GLCD_CD_OUT &= ~(1<<PORT_GLCD_CD);
	PORT_GLCD_DB = val&0xff;
	PORT_GLCD_WR_OUT &= ~(1<<PORT_GLCD_WR);
	_delay_us(1);
	PORT_GLCD_WR_OUT |=  (1<<PORT_GLCD_WR);
	//Wait until LCD is ready
	timeout=0; while (((glcd_read_byte(1)&0x03)!=0x03) && (timeout++ < 500)) {};
	PORT_GLCD_CD_OUT &= ~(1<<PORT_GLCD_CD);
	PORT_GLCD_DB = val>>8;
	PORT_GLCD_WR_OUT &= ~(1<<PORT_GLCD_WR);
	_delay_us(1);
	PORT_GLCD_WR_OUT |=  (1<<PORT_GLCD_WR);

	//Wait until LCD is ready
	timeout=0; while (((glcd_read_byte(1)&0x03)!=0x03) && (timeout++ < 500)) {};
	PORT_GLCD_CD_OUT |= (1<<PORT_GLCD_CD);
	PORT_GLCD_DB = cmd;
	PORT_GLCD_WR_OUT &= ~(1<<PORT_GLCD_WR);
	_delay_us(1);
	PORT_GLCD_WR_OUT |=  (1<<PORT_GLCD_WR);
	//
	PORT_GLCD_CE_OUT |=  (1<<PORT_GLCD_CE);
}

void glcd_write_byte(unsigned char val, unsigned char cmd) {
	unsigned int timeout = 0;
	// 
	PORT_GLCD_CE_OUT &= ~(1<<PORT_GLCD_CE);
	//Wait until LCD is ready
	while (((glcd_read_byte(1)&0x03)!=0x03) && (timeout++ < 500)) {};
	if (cmd) PORT_GLCD_CD_OUT |= (1<<PORT_GLCD_CD);
	else PORT_GLCD_CD_OUT &= ~(1<<PORT_GLCD_CD);
	PORT_GLCD_DB = val;
	PORT_GLCD_WR_OUT &= ~(1<<PORT_GLCD_WR);
	_delay_us(1);
	PORT_GLCD_WR_OUT |=  (1<<PORT_GLCD_WR);
	// 
	PORT_GLCD_CE_OUT |=  (1<<PORT_GLCD_CE);
}

unsigned char glcd_read_byte(unsigned char status) {
	unsigned char d;

	DDR_GLCD_DB = 0x00;
	// PORT_GLCD_CE_OUT &= ~(1<<PORT_GLCD_CE);
	if (status) PORT_GLCD_CD_OUT |= (1<<PORT_GLCD_CD);
		else PORT_GLCD_CD_OUT &= ~(1<<PORT_GLCD_CD);
	PORT_GLCD_RD_OUT &= ~(1<<PORT_GLCD_RD);
	_delay_us(1);
	d = PIN_GLCD_DB;
	PORT_GLCD_RD_OUT |=  (1<<PORT_GLCD_RD);
	// PORT_GLCD_CE_OUT |=  (1<<PORT_GLCD_CE);
	DDR_GLCD_DB = 0xFF;
	return d;
}

void glcd_write_buf(unsigned char *val, unsigned char len) {
	unsigned int timeout;
	glcd_write_byte(AWRON,1);
	//
	PORT_GLCD_CE_OUT &= ~(1<<PORT_GLCD_CE);
	while (len--) {
		//Wait until LCD is ready
		timeout=0; while ((!(glcd_read_byte(1)&0x08)) && (timeout++ < 500)) {};
		PORT_GLCD_CD_OUT &= ~(1<<PORT_GLCD_CD);
		PORT_GLCD_DB = *val; val++;
		PORT_GLCD_WR_OUT &= ~(1<<PORT_GLCD_WR);
		_delay_us(1);
		PORT_GLCD_WR_OUT |=  (1<<PORT_GLCD_WR);
	}
	//
	PORT_GLCD_CE_OUT |=  (1<<PORT_GLCD_CE);
	glcd_write_byte(AWROFF,1);
}

void glcd_read_buf(unsigned char *val, unsigned char len) {
	unsigned int timeout;
	glcd_write_byte(ARDON,1);
	//
	PORT_GLCD_CE_OUT &= ~(1<<PORT_GLCD_CE);
	while (len--) {
		//Wait until LCD is ready
		timeout=0; while ((!(glcd_read_byte(1)&0x04)) && (timeout++ < 500)) {};
		DDR_GLCD_DB = 0x00;
		PORT_GLCD_CD_OUT &= ~(1<<PORT_GLCD_CD);
		PORT_GLCD_RD_OUT &= ~(1<<PORT_GLCD_RD);
		_delay_us(1);
		*val = PIN_GLCD_DB; val++;
		PORT_GLCD_RD_OUT |=  (1<<PORT_GLCD_RD);
		DDR_GLCD_DB = 0xFF;
	}
	//
	PORT_GLCD_CE_OUT |=  (1<<PORT_GLCD_CE);
	glcd_write_byte(AWROFF,1);
}

void glcd_write_val(unsigned char val, unsigned int len) {
		unsigned int timeout;
	glcd_write_byte(AWRON,1);
	//
	PORT_GLCD_CE_OUT &= ~(1<<PORT_GLCD_CE);
	while (len--) {
		//Wait until LCD is ready
		timeout=0; while ((!(glcd_read_byte(1)&0x08)) && (timeout++ < 500)) {};
		PORT_GLCD_CD_OUT &= ~(1<<PORT_GLCD_CD);
		PORT_GLCD_DB = val;
		PORT_GLCD_WR_OUT &= ~(1<<PORT_GLCD_WR);
		_delay_us(1);
		PORT_GLCD_WR_OUT |=  (1<<PORT_GLCD_WR);
	}
	//
	PORT_GLCD_CE_OUT |=  (1<<PORT_GLCD_CE);
	glcd_write_byte(AWROFF,1);
}

void glcd_clrtext(void) {
	// CLEAR TEXT SCREEN
	glcd_write_cmd(ADPSET,0x0000);
	glcd_write_val(0x00,(GLCD_X_RES>>3)*(GLCD_Y_RES>>3));

	glcd_pos(0,0);
}

void glcd_clrgraphics(void) {	
	// CLEAR GRAPHICS SCREEN
	glcd_write_cmd(ADPSET,0x0200);
	glcd_write_val(0x00,(GLCD_X_RES>>3)*GLCD_Y_RES);
}

void glcd_scroll(void) {
	unsigned char buf[GLCD_X_RES>>3];
	int i;
	
	for(i=0;i<((GLCD_Y_RES>>3)-1);i++) {
		glcd_write_cmd(ADPSET,(i+1)*(GLCD_X_RES>>3));
		glcd_read_buf(buf,GLCD_X_RES>>3);
		glcd_write_cmd(ADPSET,i*(GLCD_X_RES>>3));
		glcd_write_buf(buf,GLCD_X_RES>>3);
	}
	glcd_write_cmd(ADPSET,i*(GLCD_X_RES>>3));
	glcd_write_val(0x00,(GLCD_X_RES>>3));
}

unsigned char glcd_cx;
unsigned char glcd_cy;

void glcd_pos(unsigned char x, unsigned char y) {
	
	// check right screen overflow, use next line
	while (x>=(GLCD_X_RES>>3)) { 
		x-=(GLCD_X_RES>>3); 
		y++;
	};

	// check down  screen overflow, scroll up
	while (y>=(GLCD_Y_RES>>3)) {
		y--;
		glcd_scroll();
	}
	
	// Cursor and text write position (they are completely independent!)
	glcd_write_cmd(ADPSET,y*(GLCD_X_RES>>3)+x);
	glcd_write_cmd(CURSET,(y<<8)+x);
	glcd_cx = x;
	glcd_cy = y;
}
void glcd_write_char(unsigned char c) {
	glcd_pos(glcd_cx,glcd_cy);
	if (c==0x0A) 
		glcd_cy++;
	else { if (c==0x0D) glcd_cx=0;
		   else if (c==0x08) glcd_cx--;
				else {
					glcd_cx++;
					if (c<0x20) glcd_write_byte(0x80,0);
					else glcd_write_byte(c-0x20,0);
					glcd_write_byte(DWR,1);
				}
		}		
	glcd_pos(glcd_cx,glcd_cy);
}

#if !USE_GLCD_STREAM
//When not in stream mode, we have an alternative (simple) print function.
void glcd_write_str(char *val, unsigned char len) {
	unsigned int timeout;
	glcd_pos(glcd_cx,glcd_cy);
	glcd_write_byte(AWRON,1);
	//
	PORT_GLCD_CE_OUT &= ~(1<<PORT_GLCD_CE);
	while (len--) {
		//Wait until LCD is ready
		timeout=0; while ((!(glcd_read_byte(1)&0x08)) && (timeout++ < 500)) {};
		PORT_GLCD_CD_OUT &= ~(1<<PORT_GLCD_CD);
		PORT_GLCD_DB = *val<0x20?0x80:*val-0x20; 
		PORT_GLCD_WR_OUT &= ~(1<<PORT_GLCD_WR);
		_delay_us(1);
		PORT_GLCD_WR_OUT |=  (1<<PORT_GLCD_WR);
		val++;
		glcd_cx++;
	}
	//
	PORT_GLCD_CE_OUT |=  (1<<PORT_GLCD_CE);
	glcd_write_byte(AWROFF,1);
	glcd_pos(glcd_cx,glcd_cy);
}
#endif

void glcd_setpixel(unsigned char x, unsigned char y) {
	unsigned int a = 0x200 + (x>>3)+(y*(GLCD_X_RES>>3));
	// home position, 2nd line
	glcd_write_cmd(ADPSET,a);
	glcd_write_byte(SETPIX+(7-(x&0x07)),1);
}

// Bresenhams line algorithm, implementation used from
// http://www.cs.unc.edu/~mcmillan/comp136/Lecture6/Lines.html
// Could be further optimized using 8bit types only...
void glcd_line(int x0, int y0, int x1, int y1)
{
	int dy = y1 - y0;
	int dx = x1 - x0;
	int stepx, stepy;

	if (dy < 0) { dy = -dy;  stepy = -(GLCD_X_RES>>3); } else { stepy = GLCD_X_RES>>3; }
	if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }
	dy <<= 1;
	dx <<= 1;

	// we use the "real" RAM addresses for y addressing
	y0 *= (GLCD_X_RES>>3);
	y1 *= (GLCD_X_RES>>3);
	y0 += 0x0200;
	y1 += 0x0200;
	glcd_write_cmd(ADPSET,y0+(x0>>3));
	glcd_write_byte(SETPIX+(7-x0%8),1);
	if (dx > dy) {
		int fraction = dy - (dx >> 1);
		while (x0 != x1) {
			if (fraction >= 0) {
				y0 += stepy;
				fraction -= dx;
			}
			x0 += stepx;
			fraction += dy;
			glcd_write_cmd(ADPSET,y0+(x0>>3));
			glcd_write_byte(SETPIX+(7-(x0&0x07)),1);
		}
	} else {
		int fraction = dx - (dy >> 1);
		while (y0 != y1) {
			if (fraction >= 0) {
				x0 += stepx;
				fraction -= dy;
			}
			y0 += stepy;
			fraction += dx;
			glcd_write_cmd(ADPSET,y0+(x0>>3));
			glcd_write_byte(SETPIX+(7-(x0&0x07)),1);
		}
	}
}

void glcd_rect(unsigned char x0, unsigned char y0, unsigned char x1, unsigned char y1) {
	glcd_line(x0,y0,x1,y0);
	glcd_line(x0,y1,x1,y1);
	glcd_line(x0,y0,x0,y1);
	glcd_line(x1,y0,x1,y1);
}

// Implementation of fast circle algorithm (no sin/cos required) found at:
// http://en.wikipedia.org/wiki/Midpoint_circle_algorithm
void glcd_circle(int x0, int y0, int radius)
{
    int f = 1 - radius;
    int ddF_x = 1;
    int ddF_y = -2 * radius;
    int x = 0;
    int y = radius;

    glcd_setpixel(x0, y0 + radius);
    glcd_setpixel(x0, y0 - radius);
    glcd_setpixel(x0 + radius, y0);
    glcd_setpixel(x0 - radius, y0);
 
    while(x < y)
    {
      //assert(ddF_x == 2 * x + 1);	
      ddF_x = 2 * x + 1;
      //assert(ddF_y == -2 * y);
      ddF_y = -2 * y;
      //assert(f == x*x + y*y - radius*radius + 2*x - y + 1);
      f = x*x + y*y - radius*radius + 2*x - y + 1;
      if(f >= 0) 
      {
        y--;
        ddF_y += 2;
        f += ddF_y;
      }
      x++;
      ddF_x += 2;
      f += ddF_x;    
      glcd_setpixel(x0 + x, y0 + y);
      glcd_setpixel(x0 - x, y0 + y);
      glcd_setpixel(x0 + x, y0 - y);
      glcd_setpixel(x0 - x, y0 - y);
      glcd_setpixel(x0 + y, y0 + x);
      glcd_setpixel(x0 - y, y0 + x);
      glcd_setpixel(x0 + y, y0 - x);
      glcd_setpixel(x0 - y, y0 - x);
    }
 }

#if USE_GLCD_STREAM
	#include <stdio.h>

	//This display supports libc printf with scrolling, etc.
	int glcd_putchar(char c, FILE *stream) {
		if (c==0x0a) glcd_write_char(0x0d);
		glcd_write_char(c);
		return 0;
	}
	FILE glcd_stream = FDEV_SETUP_STREAM(glcd_putchar, NULL, _FDEV_SETUP_WRITE);
#endif

#endif //USE_GLCD
