/*----------------------------------------------------------------------------
Copyright:      Wolfgang Scherr
 Author:         Wolfgang Scherr
 Remarks:        from scratch written, influenced by several web sources,
				 especially: T6963C Datasheet from TOSHIBA 
				 Irgendwann sollte ich mich selbst auf dt. oder en. Kommentare einigen :-)
 known Problems:  
 Version:        15.02.2009
 Description:    T6963C-type graphics display support

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#ifndef _GLCD_H
	#define _GLCD_H

#if !USE_GLCD
	#if USE_GLCD_STREAM
		#warning "GLCD disabled, but streaming to it enabled - check config!"
		//without GLCD, a stream also makes no sense...
		#undef USE_GLCD_STREAM
		#define USE_GLCD_STREAM 0
	#endif
#else
	//----------------------------------------------------------------------------
	
	// RAM-Size:
	
	// HY6264A has 8k x 8
	// 0x0000 - 0x1FFF
	
	#define GLCD_RAM 8192
	
	//----------------------------------------------------------------------------
	
	#define TXHOME 0x40 // SET TXT HM ADD
	#define TXAREA 0x41 // SET TXT AREA
	#define GRHOME 0x42 // SET GR HM ADD
	#define GRAREA 0x43 // SET GR AREA
	#define CURSET 0x21 // SET CURSOR ADD
	#define OFFSET 0x22 // SET OFFSET ADD
	#define ADPSET 0x24 // SET ADD PTR
	 
	#define M_OR   0x80	// OR MODE
	#define M_XOR  0x81	// XOR MODE
	#define M_AND  0x83	// AND MODE
	#define M_TA   0x84	// TEXT ATTRIBUTE MODE
	#define M_ECG  0x08	// EXT. CG RAM MODE
	 
	#define DCMD   0x90 // DISPLAY COMMAND
	#define D_CN   0x02 // CURSOR, NOBLINK
	#define D_CB   0x03 // CURSOR, BLINK
	#define DTEXT  0x04 // TEXT ON
	#define DGRA   0x08 // GRAPHICS ON
	
	#define CUR1   0xA0 // 1-line cursor
	#define CUR2   0xA1 // 2-line cursor
	#define CUR3   0xA2 // 3-line cursor
	#define CUR4   0xA3 // 4-line cursor
	#define CUR5   0xA4 // 5-line cursor
	#define CUR6   0xA5 // 6-line cursor
	#define CUR7   0xA6 // 7-line cursor
	#define CUR8   0xA7 // 8-line cursor
	
	#define AWRON  0xB0 // SET AUTO WRITE MODE
	#define ARDON  0xB1 // SET AUTO READ MODE
	#define AWROFF 0xB2 // RESET AUTO WRITE MODE

	#define DWR    0xC0 // WRITE BYTE
	#define DRD    0xC1 // READ BYTE

	#define PEEK   0xE0 // CAPTURE FROM MEMORY
	
	#define CLRPIX 0xF0 // CLEAR A PIXEL
	#define SETPIX 0xF8 // SET A PIXEL
	//----------------------------------------------------------------------------

	void glcd_init(void);
	void glcd_write_cmd(unsigned char, unsigned int);
	void glcd_write_byte(unsigned char, unsigned char);
	unsigned char glcd_read_byte(unsigned char);
	void glcd_write_buf(unsigned char *, unsigned char);
	void glcd_read_buf(unsigned char *, unsigned char);
	void glcd_write_val(unsigned char, unsigned int);

	void glcd_clrtext(void);
	void glcd_clrgraphics(void);
	
	void glcd_pos(unsigned char, unsigned char);
	
	#if !USE_GLCD_STREAM
	//When not in stream mode, we have an alternative (simple) print function.
		void glcd_write_char(unsigned char);
		void glcd_write_str(char *, unsigned char);
	#endif

	void glcd_setpixel(unsigned char x, unsigned char y);
	void glcd_line(int,int,int,int);
	void glcd_rect(unsigned char,unsigned char,unsigned char,unsigned char);
	void glcd_circle(int,int,int);

	//----------------------------------------------------------------------------

	#if USE_GLCD_STREAM
	// In stream mode, fprintf(glcd_stream,....) can be used.	
		extern FILE glcd_stream;
	#endif
	
#endif //USE_GLCD

#endif //_GLCD_H
