/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich, (Ralf Figge)  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich, reworked by Wolfgang Scherr
 Remarks:        Rework: massiv berarbeitet, jetzt viel flexibler
 known Problems: none
 Version:        12.02.2009
 Description:    Programm zur Ansteuerung eines Standard LCD
				 (HD44870),(SED1278F) und kompatible
------------------------------------------------------------------------------*/

#if USE_LCD

#ifndef _LCD_H_
	#define _LCD_H_

	//Trickreiche Konfiguration - geht nur, wenn Werte nicht kollidieren
	//Anzahl der Zeilen 1,2 oder 4
	#if (LCD_SIZE==1*16) || (LCD_SIZE==1*20)
		#define ONE_LINES
	#elif (LCD_SIZE==2*16) || (LCD_SIZE==2*20)
		#define TWO_LINES
	#elif (LCD_SIZE==4*16) || (LCD_SIZE==4*20)
		#define FOUR_LINES
	#else
		#error "LCD_SIZE inkorrekt, kann Code nicht generieren."
	#endif
	//Anzahl der Spalten 20 oder 16
	#if (LCD_SIZE==1*20) || (LCD_SIZE==2*20) || (LCD_SIZE==4*20)
		#define COL20
	#elif (LCD_SIZE==1*16) || (LCD_SIZE==2*16) || (LCD_SIZE==4*16)
		#ifdef COL20
			#undef COL20
		#endif
	#else
		#error "LCD_SIZE inkorrekt, kann Code nicht generieren."
	#endif

	#if (LCD_EXT_PORT+LCD_SER+LCD_PAR+LCD_SerLCD)>1
		#error "Es darf nur eine LCD-Konfiguration aktiviert werden!"
	#endif
	
	//Hier knnten noch mehrere "offizielle" Belegungen fix definiert werden.
	#if LCD_EXT_PORT
		// LCD-Connection am EXT Port des AVR_NET_IO lt. Pollin-Datenblatt
		#undef PORT_LCD_DATA_OUT 
		#undef PORT_LCD_DATA_IN 
		#undef DDR_LCD_DATA 
		#undef LCD_DATA_LOW_NIBBLE
		#undef PORT_LCD_E 
		#undef DDR_LCD_E 
		#undef LCD_E 
		#undef PORT_LCD_RW 
		#undef DDR_LCD_RW 
		#undef LCD_RW
		#undef PORT_LCD_RS 
		#undef DDR_LCD_RS 
		#undef LCD_RS 
		#undef PORT_LCD_BL 
		#undef DDR_LCD_BL 
		#undef LCD_BL
		#define PORT_LCD_DATA_OUT   PORTD
		#define PORT_LCD_DATA_IN    PIND
		#define DDR_LCD_DATA        DDRD
		#define LCD_DATA_LOW_NIBBLE 0
		#define PORT_LCD_E  PORTB
		#define DDR_LCD_E   DDRB
		#define LCD_E       0
		#define PORT_LCD_RW PORTD
		#define DDR_LCD_RW  DDRD
		#define LCD_RW      2
		#define PORT_LCD_RS PORTD
		#define DDR_LCD_RS  DDRD
		#define LCD_RS      3
		#define PORT_LCD_BL PORTB
		#define DDR_LCD_BL  DDRB
		#define LCD_BL      3
		// finally we enable the flag to use the parallel IF
		#undef LCD_PAR
		#define LCD_PAR 1
	#endif //LCD_EXT_PORT

	#if LCD_SerLCD
		// LCD-Connection seriell entsprechend Ulrich Radigs Webpage
		#undef PORT_LCD_DATA    
		#undef DDR_LCD_DATA     
		#undef LCD_DATA         
		#undef PORT_LCD_CLOCK   
		#undef DDR_LCD_CLOCK    
		#undef LCD_CLOCK        
		#undef LCD_RS_PIN       
		#undef LCD_BL_PIN       
		#undef LCD_DATA_MSB_PIN 
		#undef PORT_LCD_E
		#undef DDR_LCD_E
		#undef LCD_E
		#define PORT_LCD_DATA        PORTC
		#define DDR_LCD_DATA         DDRC
		#define LCD_DATA             3
		#define PORT_LCD_CLOCK       PORTC
		#define DDR_LCD_CLOCK        DDRC
		#define LCD_CLOCK            4
		#define LCD_RS_PIN       0
		#define LCD_BL_PIN       7
		#define LCD_DATA_MSB_PIN 6
		#define PORT_LCD_E  PORTC
		#define DDR_LCD_E   DDRC
		#define LCD_E       2
		// finally we enable the flag to use the serial IF
		#undef LCD_SER
		#define LCD_SER 1
	#endif
	
	//Ein paar Definitionen zur Codevereinfachung...
	#if LCD_PAR
		#if LCD_DATA_LOW_NIBBLE
			#define LCD_DATA_MASK 0x0F
			#define LCD_DATA_INV_MASK 0xF0
			#define LCD_DATA_WRITEH(val) PORT_LCD_DATA_OUT=(PORT_LCD_DATA_OUT&LCD_DATA_INV_MASK)|(((val)>>4)&LCD_DATA_MASK)
			#define LCD_DATA_WRITEL(val) PORT_LCD_DATA_OUT=(PORT_LCD_DATA_OUT&LCD_DATA_INV_MASK)|((val)&LCD_DATA_MASK)
			#define LCD_DATA_READH ((PORT_LCD_DATA_IN&LCD_DATA_MASK)<<4)
			#define LCD_DATA_READL (PORT_LCD_DATA_IN&LCD_DATA_MASK)
		#else
			#define LCD_DATA_MASK 0xF0	
			#define LCD_DATA_INV_MASK 0x0F
			#define LCD_DATA_WRITEH(val) PORT_LCD_DATA_OUT=(PORT_LCD_DATA_OUT&LCD_DATA_INV_MASK)|((val)&LCD_DATA_MASK)
			#define LCD_DATA_WRITEL(val) PORT_LCD_DATA_OUT=(PORT_LCD_DATA_OUT&LCD_DATA_INV_MASK)|(((val)<<4)&LCD_DATA_MASK)
			#define LCD_DATA_READH (PORT_LCD_DATA_IN&LCD_DATA_MASK)
			#define LCD_DATA_READL ((PORT_LCD_DATA_IN&LCD_DATA_MASK)>>4)
		#endif
		#define LCD_RSH PORT_LCD_RS |= (1<<LCD_RS)
		#define LCD_RSL PORT_LCD_RS &=~ (1<<LCD_RS)
		#define LCD_RWH PORT_LCD_RW |= (1<<LCD_RW)
		#define LCD_RWL PORT_LCD_RW &=~ (1<<LCD_RW)
	#endif
	
	#if LCD_SER
		#define LCD_DATA_MASK (0xF0>>(7-LCD_DATA_MSB_PIN))
		#define LCD_DATA_INV_MASK (~LCD_DATA_MASK)
		#define LCD_DATA_WRITEH(val) lcd_shiftreg=(lcd_shiftreg&LCD_DATA_INV_MASK)|(((val)>>(7-LCD_DATA_MSB_PIN))&LCD_DATA_MASK);shift_data(lcd_shiftreg)
		#define LCD_DATA_WRITEL(val) lcd_shiftreg=(lcd_shiftreg&LCD_DATA_INV_MASK)|(((val)<<(LCD_DATA_MSB_PIN-3))&LCD_DATA_MASK);shift_data(lcd_shiftreg)
		#define LCD_RSH lcd_shiftreg |= (1<<LCD_RS_PIN)
		#define LCD_RSL lcd_shiftreg &=~ (1<<LCD_RS_PIN)
		#define LCD_BLH lcd_shiftreg |= (1<<LCD_BL_PIN);shift_data(lcd_shiftreg)
		#define LCD_BLL lcd_shiftreg &=~ (1<<LCD_BL_PIN);shift_data(lcd_shiftreg)
		#define LCD_RWL 
 	#endif

	//Wir bleiben auf der sicheren Seite und warten ein bischen... :-)
	#define LCD_EH PORT_LCD_E |= (1<<LCD_E);_delay_us(1)
	#define LCD_EL PORT_LCD_E &=~ (1<<LCD_E)

	//Interne Prototypen
	#if LCD_PAR	&& LCD_PAR_READ
		#define BUSYBIT	7
		char lcd_read (char);
	#endif
	#if LCD_SER
		void shift_data(char);
	#endif
	//Prototypes
	extern void lcd_write (char,char);

	extern void lcd_init (void);
	extern void lcd_pos (unsigned char,unsigned char);
	extern void lcd_clear (void);
	#define lcd_print_char(a) lcd_write(a,1)

	#if !USE_LCD_STREAM
		extern void lcd_print_str (char *Buffer);
		extern void lcd_print_P (unsigned char,unsigned char,const char *Buffer,...);
		#define lcd_print(a,b,format, args...)  lcd_print_P(a,b,PSTR(format) , ## args)
	#else
	// In stream mode, fprintf(glcd_stream,....) can be used.	
		extern FILE lcd_stream;
	#endif
	
	#ifdef USE_LCD_BACKLIGHT
		extern void lcd_backlight (char);
	#endif
	
#endif //_LCD_H_

#endif //USE_LCD
