/*----------------------------------------------------------------------------
 Copyright:      Wolfgang Scherr
 Author:         Wolfgang Scherr
 Remarks:        from scratch written, influenced by several web sources,
				 especially: http://www.beyondlogic.org/keyboard/keybrd.htm 
				 and AVR313 also got a nice english kb. translation table from
				 http://www.htw-dresden.de/~beck/Atmel/Keyboard/PS_2-keyboard.html
				 Irgendwann sollte ich mich selbst auf dt. oder en. Kommentare einigen :-)
 known Problems: may cause garbled output for some keys, still "alpha" code 
 Version:        15.02.2009
 Description:    PS2 Keyboard Support, with self-recoverable clock sync

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"

#if USE_PS2KB

	#include <avr/io.h>
	#include <avr/interrupt.h>
	#include <avr/pgmspace.h>

#if USE_PS2KB_STREAM
	#include <stdio.h>
#endif

	#include "timer.h"

	#include "ps2_kb.h"

KB_ITEM kb_status;

unsigned char kb_rx_buffer;

volatile unsigned int bitcnt = 0;
volatile unsigned int bufcnt = 0;
unsigned char shift = 0;
char buffer[8];

volatile unsigned char kb_timeout = 0;

//QUERTY keyboard
#if PS2KB_ENGLISH
    char PROGMEM CodingUnshifted[]=
   {/*   0    1    2    3    4    5    6    7    8    9    a    b    c    d    e   f*/
/*0*/    0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,'^' ,0   ,
/*1*/    0   ,0   ,0   ,0   ,0   ,'q' ,'1' ,0   ,0   ,0   ,'z' ,'s' ,'a' ,'w' ,'2' ,0   ,
/*2*/    0   ,'c' ,'x' ,'d' ,'e' ,'4' ,'3' ,0   ,0   ,' ' ,'v' ,'f' ,'t' ,'r' ,'5' ,0   ,
/*3*/    0   ,'n' ,'b' ,'h' ,'g' ,'y' ,'6' ,0   ,0   ,',' ,'m' ,'j' ,'u' ,'7' ,'8' ,0   ,
/*4*/    0   ,',' ,'k' ,'i' ,'o' ,'0' ,'9' ,0   ,0   ,'.' ,'?' ,'l' ,':' ,'p' ,'-' ,0   ,
/*5*/    0   ,0   ,'\'',0   ,'[' ,'+' ,0   ,0   ,0   ,0   ,13  ,']' ,0   ,'|', 0   ,0   ,
/*6*/    0   ,'<' ,0   ,0   ,0   ,0   ,8   ,0   ,0   ,'1' ,0   ,'4' ,'7' ,0   ,0   ,0   ,
/*7*/    '0' ,',' ,'2' ,'5' ,'6' ,'8' ,0x1b,0   ,0   ,'+' ,'3' ,'-' ,'*' ,'9' ,0   ,0  
   };
   char PROGMEM CodingShifted[]=
   {
/*0*/     0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,9   ,0   ,0   ,
/*1*/     0   ,0   ,0   ,0   ,4   ,'Q' ,'!' ,0   ,0   ,0   ,'Z' ,'S' ,'A' ,'W' ,'@' ,0   ,
/*2*/     0   ,'C' ,'X' ,'D' ,'E' ,'$' ,'#' ,0   ,0   ,' ' ,'V' ,'F' ,'T' ,'R' ,'%' ,0   ,
/*3*/     0   ,'N' ,'B' ,'H' ,'G' ,'Y' ,'^' ,0   ,0   ,',' ,'M' ,'J' ,'U' ,'&' ,'*' ,0   ,
/*4*/     0   ,'<' ,'K' ,'I' ,'O' ,')' ,'(' ,0   ,0   ,'>' ,'/' ,'L' ,';' ,'P' ,'_' ,0   ,
/*5*/     0   ,0   ,'"' ,0   ,'{' ,'=' ,0   ,0   ,0   ,0   ,13  ,'}' ,0   ,'\\',0   ,0   ,
/*6*/     0   ,'>' ,0   ,0   ,0   ,0   ,8   ,0   ,0   ,'1' ,0   ,'4' ,'7' ,0   ,0   ,0   ,
/*7*/     '0' ,';' ,'2' ,'5' ,'6' ,'8' ,0x1b,0   ,0   ,'+' ,'3' ,'-' ,'*' ,'9' ,0   ,0  
   };
#endif

//QUERTZ keyboard, created from above...
#if PS2KB_GERMAN
    char PROGMEM CodingUnshifted[]=
   {/*   0    1    2    3    4    5    6    7    8    9    a    b    c    d    e   f*/
/*0*/    0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,'^' ,0   ,
/*1*/    0   ,0   ,0   ,0   ,0   ,'q' ,'1' ,0   ,0   ,0   ,'y' ,'s' ,'a' ,'w' ,'2' ,0   ,
/*2*/    0   ,'c' ,'x' ,'d' ,'e' ,'4' ,'3' ,0   ,0   ,' ' ,'v' ,'f' ,'t' ,'r' ,'5' ,0   ,
/*3*/    0   ,'n' ,'b' ,'h' ,'g' ,'z' ,'6' ,0   ,0   ,',' ,'m' ,'j' ,'u' ,'7' ,'8' ,0   ,
/*4*/    0   ,',' ,'k' ,'i' ,'o' ,'0' ,'9' ,0   ,0   ,'.' ,'-' ,'l' ,148 ,'p' ,225 ,0   ,
/*5*/    0   ,0   ,132 ,0   ,129 ,'`' ,0   ,0   ,0   ,0   ,13  ,'+' ,0   ,'#' , 0   ,0   ,
/*6*/    0   ,'<' ,0   ,0   ,0   ,0   ,8   ,0   ,0   ,'1' ,0   ,'4' ,'7' ,0   ,0   ,0   ,
/*7*/    '0' ,',' ,'2' ,'5' ,'6' ,'8' ,27  ,0   ,0   ,'+' ,'3' ,'-' ,'*' ,'9' ,0   ,0  
   };
   char PROGMEM CodingShifted[]=
   {
/*0*/     0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,0   ,9   ,167 ,0   ,
/*1*/     0   ,0   ,0   ,0   ,4   ,'Q' ,'!' ,0   ,0   ,0   ,'Y' ,'S' ,'A' ,'W' ,'\"',0   ,
/*2*/     0   ,'C' ,'X' ,'D' ,'E' ,'$' ,'\\',0   ,0   ,' ' ,'V' ,'F' ,'T' ,'R' ,'%' ,0   ,
/*3*/     0   ,'N' ,'B' ,'H' ,'G' ,'Z' ,'&' ,0   ,0   ,',' ,'M' ,'J' ,'U' ,'/' ,'(' ,0   ,
/*4*/     0   ,';' ,'K' ,'I' ,'O' ,'=' ,')' ,0   ,0   ,':' ,'_' ,'L' ,153 ,'P' ,'?' ,0   ,
/*5*/     0   ,0   ,142 ,0   ,154 ,'\'' ,0  ,0   ,0   ,0   ,13  ,'*' ,0   ,'\'',0   ,0   ,
/*6*/     0   ,'>' ,0   ,0   ,0   ,0   ,8   ,0   ,0   ,'1' ,0   ,'4' ,'7' ,0   ,0   ,0   ,
/*7*/     '0' ,';' ,'2' ,'5' ,'6' ,'8' ,27  ,0   ,0   ,'+' ,'3' ,'-' ,'*' ,'9' ,0   ,0  
   };
#endif

//----------------------------------------------------------------------------
//Init PS2/AT Keyboard Interface
void kb_init() { 
  	//Enable Interrupt
	#ifdef INTPOL
		INTPOL |= BIT_INTPOL;
	#endif
	INTREG |= BIT_INTEN;
	kb_rx_buffer=0;
	bitcnt=0;
	kb_status.kb_ready=0;
	
	//Timeout to reset bitcounter after 2sec
	add_timer_flag((unsigned char *)&kb_timeout,2);
}

//----------------------------------------------------------------------------
//Empfang eines Zeichens
ISR (PS2KB_CLKINT)
{
	//After 2sec non-activity, we always assume a fresh start - 
	//this keeps the kb-timing in-sync
	if (kb_timeout) {
		bitcnt=0; bufcnt=0; shift=0;
	}
	kb_timeout=0;
		
	#ifndef INTPOL
		//Wir haben keine HW-Flankenerkennung
		if (!(PS2KB_CLKPIN&(1<<PS2KB_CLK))) return;
	#endif

	if (bitcnt<9) {
		//Das Startbit schieben wir einfach ber die 8 Bit hinaus... 
		buffer[bufcnt] >>= 1; 
		buffer[bufcnt] |= (PS2KB_DATAPIN&(1<<PS2KB_DATA))?0x80:0;
		bitcnt++;
	} else if (bitcnt==9) {
		//Wir sparen uns die Parityabfrage, und berspringen das Bit
		bitcnt++;
	} else if (bitcnt>9) {
		//Jetzt haben wir ein einen neuen Scancode
		bitcnt=0;
		if (buffer[0]==0xF0) {
			// key released code finalized 
			if (bufcnt==1) {
				buffer[0]=0;
				bufcnt=0;
				if ((buffer[1]==0x12) || (buffer[1]==0x59))
					shift=0;
			}
			else
				bufcnt++;
		}
		else if (buffer[0]==0xE1) {
			// we ignore Pause/Brk (8 byte)
			if (bufcnt==7) 
				bufcnt=0;
			else
				bufcnt++;
		}
		else if (buffer[0]==0xE0) {
			// we ignore Prt/Scr (4 byte)
			if (buffer[1]==0x12) {
				if (bufcnt==3) {
					bufcnt=0;
					buffer[1]=0;
				} else
					bufcnt++;
			// we ignore extended commands (2 byte)
			} else { 
				if (bufcnt==1) 
					bufcnt=0;
				else
					bufcnt++;
			}
		}
		else {
			// we look up all standard keys + shift
			if ((buffer[0]==0x12) || (buffer[0]==0x59))
				shift=1;
			else if (buffer[0]<0x80) {
				if(!shift)                    // If shift not pressed,
					kb_rx_buffer=(char)pgm_read_byte(&(CodingUnshifted[(int)buffer[0]]));
				else
					kb_rx_buffer=(char)pgm_read_byte(&(CodingShifted  [(int)buffer[0]]));
				kb_status.kb_ready=1;
			}
			bufcnt=0;
		}
	} else
		bitcnt++;
}

#if USE_PS2KB_STREAM
	#include <stdio.h>

	//Check for data available;
	char kb_ischar(void) {
		return kb_status.kb_ready;
	}

	//This display supports libc scanf etc.
	int kb_getchar(FILE *stream) {
		char c;
		while(!kb_ischar());
		c=kb_rx_buffer;
		kb_status.kb_ready=0;
		return c;
	}
	FILE kb_stream = FDEV_SETUP_STREAM(NULL, kb_getchar, _FDEV_SETUP_READ);
#endif

#endif //USE_PS2KB
