/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich, reworked by Wolfgang Scherr
 Remarks:        Rework: config.h, unschoene Codeabhngigkeiten entfernt,
				 komplett neues Puffer-Handling ohne Pointer, Streaming-
				 Handling fr libc-Funktionen (damit redundante write-Fkt. weg)
 known Problems: none
 Version:        19.03.2009
 Description:    RS232 Routinen

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/


#include "config.h"

#if USE_USART

	#include <avr/io.h>
	#include <avr/interrupt.h>
	#include <stdlib.h>
	
#if !USE_USART_STREAM
	#include <avr/pgmspace.h>
	#include <stdarg.h>
	#include <ctype.h>
	#include <string.h>
#else
	#include <stdio.h>
#endif

	#include "usart.h"

	//----------------------------------------------------------------------------

volatile unsigned char usart_buf_in = 0;
volatile unsigned char usart_buf_out = 0;

char usart_rx_buffer[USART_BUFFER_SIZE];
	
//----------------------------------------------------------------------------
//Init serielle Schnittstelle
void usart_init() { 
	//Serielle Schnittstelle 1
	
  	//Enable TXEN/RXEN & RX Interrupt
	UCR =(1 << TXEN | 1 << RXEN | 1<< RXCIE);
	//Teiler wird gesetzt 
	UBRR=UBRR_VAL & 0xFF;
}

//----------------------------------------------------------------------------
//Routine fr die Serielle Ausgabe eines Zeichens (Schnittstelle0)
void usart_write_char(char c)
{
	//Warten solange bis Zeichen gesendet wurde
	while(!(USR & (1<<UDRE)));
	//Ausgabe des Zeichens
	UDR = c;
}

#if !USE_USART_STREAM
//------------------------------------------------------------------------------
void usart_write_P (const char *Buffer,...)
{
	va_list ap;
	va_start (ap, Buffer);	
	
	int format_flag;
	char str_buffer[10];
	char str_null_buffer[10];
	char move = 0;
	char Base = 0;
	int tmp = 0;
	char by;
	char *ptr;
		
	//Ausgabe der Zeichen
    for(;;)
	{
		by = pgm_read_byte(Buffer++);
		if(by==0) break; // end of format string
            
		if (by == '%')
		{
            by = pgm_read_byte(Buffer++);
			if (isdigit(by)>0)
				{
                                 
 				str_null_buffer[0] = by;
				str_null_buffer[1] = '\0';
				move = atoi(str_null_buffer);
                by = pgm_read_byte(Buffer++);
				}

			switch (by)
				{
                case 's':
                    ptr = va_arg(ap,char *);
                    while(*ptr) { usart_write_char(*ptr++); }
                    break;
				case 'b':
					Base = 2;
					goto ConversionLoop;
				case 'c':
					//Int to char
					format_flag = va_arg(ap,int);
					usart_write_char (format_flag++);
					break;
				case 'i':
					Base = 10;
					goto ConversionLoop;
				case 'o':
					Base = 8;
					goto ConversionLoop;
				case 'x':
					Base = 16;
					//****************************
					ConversionLoop:
					//****************************
					itoa(va_arg(ap,int),str_buffer,Base);
					int b=0;
					while (str_buffer[b++] != 0){};
					b--;
					if (b<move)
						{
						move -=b;
						for (tmp = 0;tmp<move;tmp++)
							{
							str_null_buffer[tmp] = '0';
							}
						str_null_buffer[tmp] = '\0';
						strcat(str_null_buffer,str_buffer);
						strcpy(str_buffer,str_null_buffer);
						}
					usart_write_str (str_buffer);
					move =0;
					break;
				}
			
			}	
		else
		{
			usart_write_char ( by );	
		}
	}
	va_end(ap);
}

//----------------------------------------------------------------------------
//Ausgabe eines Strings
void usart_write_str(char *str)
{
	while (*str)
	{
		usart_write_char(*str++);
	}
}
#endif

//----------------------------------------------------------------------------
//Empfang eines Zeichens
ISR (USART_RX)
{
	unsigned char receive_char = UDR;
	
	if ((usart_buf_in==(usart_buf_out+1)) || (usart_buf_out==(USART_BUFFER_SIZE-1) && usart_buf_in==0)) {
		// OVL!!!
		return;
	}
	
	#if USART_ECHO
		usart_write_char(receive_char);
	#endif
	
	usart_rx_buffer[usart_buf_out++] = receive_char;

	if (usart_buf_out==USART_BUFFER_SIZE) usart_buf_out=0;
}

#if USE_USART_STREAM
	//This display supports libc printf with scrolling, etc.
	int usart_putchar(char c, FILE *stream) {
		if (c==0x0a) usart_write_char(0x0d);
		usart_write_char(c);
		return 0;
	}

	//Check for data available;
	char usart_ischar(void) {
		return (usart_buf_in!=usart_buf_out);
	}

	//This display supports libc scanf etc.
	int usart_getchar(FILE *stream) {
		char c;
		while(!usart_ischar());
		c=usart_rx_buffer[usart_buf_in++];
		if (usart_buf_in==USART_BUFFER_SIZE) usart_buf_in=0;

		return c;
	}
	FILE usart_stream = FDEV_SETUP_STREAM(usart_putchar, usart_getchar, _FDEV_SETUP_RW);
#endif

#endif //USE_USART
