/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich, reworked by Wolfgang Scherr
 Remarks:        Rework: config.h, unschoene Codeabhngigkeiten entfernt,
				 komplett neues Puffer-Handling ohne Pointer, Streaming-
				 Handling fr libc-Funktionen (damit redundante write-Fkt. weg)
 known Problems: none
 Version:        19.03.2009
 Description:    RS232 Routinen

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#ifndef _USART_H
	#define _USART_H

#if !USE_USART
	#if USE_USART_STREAM
		#warning "UART disabled, but streaming to it enabled - check config!"
		//without UART, a stream also makes no sense...
		#undef USE_USART_STREAM
		#define USE_USART_STREAM 0
	#endif
#else

	//----------------------------------------------------------------------------
		
	#ifndef F_CPU
		#error "Keine Taktfrequenz (F_CPU) definiert!"
	#endif

	#ifndef BAUDRATE
		#error "Keine Baudrate (BAUDRATE) definiert!"
	#endif

	// Berechnungen
	#define UBRR_VAL ((F_CPU)/(BAUDRATE*16L)-1)
	#if (UBRR_VAL>255)
		#error "Taktrate und Baudrate passen nicht zusammen!"
	#endif

	//Anpassen der seriellen Schnittstellen Register
	#if defined (__AVR_ATmega128__)
		#define USR UCSR0A
		#define UCR UCSR0B
		#define UDR UDR0
		#define UBRR UBRR0L
		#define USART_RX USART0_RX_vect 
	#endif
	
	#if defined (__AVR_ATmega644__) || defined (__AVR_ATmega644P__)
		#define USR UCSR0A
		#define UCR UCSR0B
		#define UBRR UBRR0L
		#define EICR EICRB
		#define TXEN TXEN0
		#define RXEN RXEN0
		#define RXCIE RXCIE0
		#define UDR UDR0
		#define UDRE UDRE0
		#define USART_RX USART0_RX_vect   
	#endif
	
	#if defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)
		#define USR UCSRA
		#define UCR UCSRB
		#define UBRR UBRRL
		#define EICR EICRB
		#define USART_RX USART_RXC_vect  
	#endif
	
	#if defined (__AVR_ATmega8__)
		#define USR UCSRA
		#define UCR UCSRB
		#define UBRR UBRRL
	#endif
	
	#if defined (__AVR_ATmega88__)
		#define USR UCSR0A
		#define UCR UCSR0B
		#define UBRR UBRR0L
		#define TXEN TXEN0
		#define UDR UDR0
		#define UDRE UDRE0
	#endif
	
	#ifndef USR
		#error "Prozessortyp vom Usart-Code (noch) nicht untersttzt!"
	#endif
	//----------------------------------------------------------------------------
	
	void usart_init(void); 
	
	#if !USE_USART_STREAM
		void usart_write_char(char c);
		void usart_write_str(char *str);
		void usart_write_P (const char *Buffer,...);
		#define usart_write(format, args...)   usart_write_P(PSTR(format) , ## args)
	#endif
	
	//----------------------------------------------------------------------------

	#if USE_USART_STREAM
	// In stream mode, fprintf(&uart_stream,....) and fscanf(&uart_stream,....) can be used.	
		extern FILE usart_stream;
		char usart_ischar(void);
	#endif
	
#endif //USE_USART

#endif //_USART_H
