/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich, reworked by Wolfgang Scherr
 Remarks:        Rework: config.h, unschoene Codeabhngigkeiten entfernt
 known Problems: none
 Version:        12.02.2009
 Description:    Webserver uvm.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/
#include "config.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "timer.h"
#include "usart.h"
#include "lcd.h"
#include "glcd.h"
#include "ps2_kb.h"

#include "analog.h"

#include "flashfs.h"

//Ethernet stack/driver
#include "stack.h"
#include "drivers/enc28j60.h"
#include "drivers/rtl8019.h"

#include "dhcpc.h"
#include "dnsc.h"
#include "ntpc.h"
#include "telnetd.h"
#include "httpd.h"

#if USE_HTTPD
//Minimalseite (sollte durch eine "richtige" Seite ersetzt werden.)
PROGMEM char Page_IDX[] = {
	"<html><title>AVR Mini-Webserver</title>\n"
	"<body>This is the default webpage.\n"
	"<form method=\"post\" action=\"/\">\n"
	"<input type=\"submit\" name=\""POST_ENDMARKER"\" value=\"POST TEST\">\n"
	"</body></html>"};

//Liefere einen Inhaltstyp fr den Dateinamen oder 0, wenn nichts gefunden.
char httpd_filetype(char *f,char *t) {
	#if MAIN_DEBUG
		printf_P(PSTR("\nType of '%s'\n"),f);
	#endif
	
	//We remove possible hierarchy separators.
	while (f[0]=='/') strcpy(f,&f[1]);

	// Process file here!

	//Default behavior...
	if (f[0]==0) {
		printf_P(PSTR("--> Asks for Index Page!\n"));
		strcpy_P(t,PSTR("text/html\0"));
		return 1;
	}

	// Not found!
	return 0;
}

//Liefere ein Datensegment (und die Lnge) fr den Dateinamen oder 0, wenn nichts gefunden.
unsigned int httpd_fileread(char *b,char *f,unsigned int start,unsigned int maxlen) {
	#if MAIN_DEBUG
		printf_P(PSTR("Read '%s' from %d (%d)\n"),f,start,maxlen);
	#endif

	//We remove possible hierarchy separators.
	while (f[0]=='/') strcpy(f,&f[1]);

	// Process file here!

	//Default behavior...
	if (f[0]==0) {
		strncpy_P(b,Page_IDX,maxlen);
		return sizeof(Page_IDX)-1;
	}
	return 0;
}

//Evaluiere und exekutiere Inhalt von POST-Inhalten.
char httpd_execute(char incomplete, char *n, char *v) {
	#if MAIN_DEBUG
		printf_P(PSTR("Execute '%s'='%s'\n"),n,v);
	#endif

	// Check end marker!
	if (strncmp_P(n,PSTR(POST_ENDMARKER),3)) return 1;
	// It is not the end marker, there must be more!
    if (incomplete) return 0;
	
	// Process data (name=n, value=n) here!

	return 0;
}
///Requests a file type for a given filename
char(*file_type_p)(char *, char *) = &httpd_filetype;
///Executes a given variable with given content
char(*exec_var_p)(char, char *,char *) = &httpd_execute;
///Reads a data segment from a given filename
unsigned int (*file_read_p)(char *buf,char *filename,unsigned int start,unsigned int maxlen) = &httpd_fileread;

#endif //USE_HTTPD

//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void)
{ 
	//"Schnelle" Konfiguration der Ausgnge bzw. Eingnge
	//Definition erfolgt in der config.h
	PORTA = SETA; DDRA = OUTA;
	PORTB = SETB; DDRB = OUTB;
	PORTC = SETC; DDRC = OUTC;
	PORTD = SETD; DDRD = OUTD;

	pre_HW_init();
		
	#if USE_TIMER
		timer_init(); //Timer initialisieren, muss am Anfang stehen!
	#endif
	
	#if USE_USART
		usart_init(); //setup the UART
	#endif

	#if USE_PS2KB
		kb_init();	  //setup AT keyboard
	#endif

	#if USE_ADC
		ADC_Init();   //setup the ADC capturing
	#endif

	#if USE_LCD
		lcd_init();	  //setup the LCD
		lcd_clear();
	#endif
	
	#if USE_GLCD
		glcd_init();  //setup the graphics LCD
		glcd_clrgraphics();
		glcd_clrtext();
	#endif //USE_GLCD

	#if USE_ETHERNET
		stack_init(); //setup/start TCP/IP stack
	#endif

	//Ethernetcard Interrupt enable
	#if USE_ETHERNET
		ETH_INT_ENABLE;
	#endif

	//Globale Interrupts einschalten
	sei(); 

	#if MAIN_DEBUG
	int a;
	printf_P(PSTR("\n\nAVR Booting....\nMemtest: "));
	//Der test sagt nicht, wieviel "wirklich" frei ist - aber
	//wenn nichts reserviert werden kann ist das auf jeden Fall schlecht...
	for (a=4096;a>0;a--) {
		void *x=malloc(a);
		if (x) {
			free(x);
			printf_P(PSTR("%d bytes SRAM free\n"),a);
			break;
		}
	}
	if (a<128) printf_P(PSTR("    --- !! MEMORY LOW !! ---\n"));
	_delay_ms(2000);
	#endif

    #if USE_DHCP
		#if MAIN_DEBUG
		printf_P(PSTR("\n\nDHCP..."));
		#endif
		dhcp_init();
		if ( dhcp() == 0)
		{
			// DHCP OK, IP set up!
			#if MAIN_DEBUG
				printf_P(PSTR("setup ok\n"));
			#endif
		}
		else
		{
			// DHCP NOK
			#if MAIN_DEBUG
				printf_P(PSTR("setup failed, default used\n"));
			#endif
			_delay_ms(5000);
			//RESET();  // this might be a little bit brutal :-)
		}
	#endif //USE_DHCP

    #if USE_DNS
		#if MAIN_DEBUG
			printf_P(PSTR("DNS init...\n"));
		#endif
		//DNS client am TCP/IP-Stack anmelden
        dns_init();
	#endif

	//Wir initialisieren das FFS, um "Beschreiben" zu ermglichen.
	#if USE_FLASHFS
		ffs_init();
	#endif

	pre_APP_init();

	#if USE_NTP	
		#if MAIN_DEBUG
			printf_P(PSTR("NTP init...\n"));
		#endif
		//NTP client am TCP/IP-Stack anmelden
		ntp_init();
		//Zeit aktualisieren    
		ntp();
	#endif //USE_NTP;

	#if USE_TELNET
		#if MAIN_DEBUG
			printf_P(PSTR("TELNETD init...\n"));
		#endif
		//Telnet-Daemon starten
		telnetd_init();
	#endif

	//Applikationen starten
	#if USE_HTTPD
		#if MAIN_DEBUG
			printf_P(PSTR("HTTPD init...\n"));
		#endif
		httpd_init(file_type_p,exec_var_p,file_read_p);
	#endif //USE_HTTPD

	/*
	udp_lcd_init();
	*/

	/*
    #if USE_MAIL
        mail_client_init();
	#endif //USE_MAIL  
	*/		

	/*	
	#if USE_SERVO
		servo_init (); //setup CAM-Servo
	#endif //USE_SERVO
	*/	

	/*
	#if USE_CAM
		cam_init();
		max_bytes = cam_picture_store(CAM_RESOLUTION);
	#endif //USE_CAM
	*/

	post_init();

	while(1)
	{
		// TCP/IP check
	    eth_get_data();

        #if USE_DHCP
			if ( dhcp() != 0) //check for lease timeout
			{
				#if MAIN_DEBUG
					printf_P(PSTR("DHCP lease renewal failed\n"));
				#endif
				//RESET();  // this might be a little bit brutal :-)
			}
		#endif //USE_DHCP

		#if USE_NTP
			//Update von Zeitinformationen, einfach regelmig aufrufen
			ntp_refresh();
		#endif //USE_NTP

		// Hauptprogramm laufen lassen
		do_main_task();
		
    }//while (1)
		
return(0);
}

