/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich, reworked by Wolfgang Scherr
 Remarks:        Rework: config.h, alle Codeabhngigkeiten entfernt:
				 Zeitgesteuerte Flags & API zur "allgemeinen" Bedienung,
				 Watchdogs zum Auslsen von Neustarts
 known Problems: none
 Version:        14.02.2009
 Description:    Timer Routinen mit 1 Sekunde Auflsung und Watchdog-
				 Funktionalitten

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"

#if USE_TIMER

	#include <avr/io.h>
	#include <avr/interrupt.h>

	#include "timer.h"

volatile unsigned long time;
TFLAG_ITEM tflags[MAX_TIMER_ENTRIES];

//----------------------------------------------------------------------------
//Diese Routine startet und inizialisiert den Timer
void timer_init (void)
{
	#if EXTCLOCK==1
		#if defined (__AVR_ATmega644__)
			//Asynchroner Modus ein, Oszillator an TOSC1 und TOSC2 aktiv
			ASSR |= (1<<AS2);
			TCCR2B = 0x05;
			while(ASSR & 0x11);
			//Capture/Compare-Interrupt aktiv
			TIMSK2 |= (1<<OCIE2A);
		#else
			//Asynchroner Modus ein, Oszillator an TOSC1 und TOSC2 aktiv
			ASSR  = (1<<AS2);
			//CTC-Modus an (Clear Timer on Compare Match)
			TCCR2 = (1<<WGM21);			
			//dieser Wert ergibt eine Sekunde Periodendauer
			OCR2  = 31;	
			//lsche Prescaler 2				
			SFIE  = (1<<PSR2);			
			//Starte Timer 2 mit Prescaler gleich 1/1024
			TCCR2 |= (1<<CS22)|(1<<CS21)|(1<<CS20); 
			while(ASSR & 0x07);
			//Capture/Compare-Interrupt aktiv
			TIMSK = (1<<OCIE2);			
		#endif
	#else
		//clear timer on compare match (CTC), 1024:1 prescaler
		TCCR1B |= (1<<WGM12) | (1<<CS10 | 0<<CS11 | 1<<CS12);
		TCNT1 = 0;
		//set reload to F_CPU/1024 to get 1 sec interrupt intervals 
		OCR1A = (F_CPU / 1024) - 1;
		//enable interrupt on compare match
		TIMSK |= (1 << OCIE1A);
	#endif

	//init timer entries
	for(int i=0;i<MAX_TIMER_ENTRIES;i++) {
		tflags[i].flag=0;
		tflags[i].wd_id=0;
	}		
	
	return;
};

//----------------------------------------------------------------------------
//Flag/Value Handler

int add_timer_flag (unsigned char *var,unsigned long time) {
	int i;
	for(i=0;i<MAX_TIMER_ENTRIES;i++) {
		if ((tflags[i].flag==0) && (tflags[i].wd_id==0)) {
			tflags[i].reload=time;
			tflags[i].cnt=time;
			tflags[i].flag=var;
			return i;
		}
	}
	return -1;
} 

void del_timer_flag (int pos) {
	if (pos>=0)	tflags[pos].flag=0;
} 

//----------------------------------------------------------------------------
//Watchdog Handler

int add_watchdog (unsigned char id, unsigned long time) {
	int i;
	//check if ID is used
	for(i=0;i<MAX_TIMER_ENTRIES;i++) {
		if (tflags[i].wd_id==id) return -1;
	}
	//find free location and set up WD
	for(i=0;i<MAX_TIMER_ENTRIES;i++) {
		if ((tflags[i].flag==0) && (tflags[i].wd_id==0)) {
			tflags[i].reload=time;
			tflags[i].cnt=time;
			tflags[i].wd_id=id;
			return i;
		}
	}
	return -1;
}

void reset_watchdog(unsigned char id) {
	int i;
	//find ID and reset value
	for(i=0;i<MAX_TIMER_ENTRIES;i++) {
		if (tflags[i].wd_id==id) {
			tflags[i].cnt=tflags[i].reload;
			return;
		}
	}
}

//----------------------------------------------------------------------------
//Timer Interrupt
#if EXTCLOCK
	#if defined (__AVR_ATmega644__)
    ISR (TIMER2_COMPA_vect) {
	#else
    ISR (TIMER2_COMP_vect) {
	#endif
#else
	ISR (TIMER1_COMPA_vect) {
#endif
	int i;

	//tick 1 second
	time++;

	//update flags
	for(i=0;i<MAX_TIMER_ENTRIES;i++) {
		if ((tflags[i].flag) || (tflags[i].wd_id)) {
			tflags[i].cnt--;
			if (! tflags[i].cnt) {
				if (tflags[i].wd_id) {
					RESET();
				} else {
					tflags[i].cnt=tflags[i].reload;
					*(tflags[i].flag)=1;
				}
			}
		}
	}
/*
	#if USE_NTP
		ntp_timer--;
	#endif //USE_NTP
	#if USE_DHCP
		if ( dhcp_lease > 0 ) dhcp_lease--;
		if ( gp_timer   > 0 ) gp_timer--;
    #endif //USE_DHCP
*/
}

#endif //USE_TIMER

