/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: POST with data content over more than two TCP 
				 packages may work not correctly.
 Version:        24.10.2007, angepasst am 21.2.2009 von WoPe fr AVR-NET-IO
 Description:    Webserver Applikation

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"

#if USE_HTTPD

    #include <stdio.h>
    #include <string.h>

	#include "stack.h"

	#include "base64.h"
	#include "httpd.h"

	#if HTTP_DEBUG
		#warning "Debugging of HTTP daemon"
		#define HTTP_DEBUGP	printf 
	#else
		#define HTTP_DEBUGP(...)	
	#endif

///This array keeps the actual HTTP requests.
HTTP_ITEM http_entry[MAX_TCP_ENTRY];

///HTTP Header: 200 OK
PROGMEM char http_header_200[]={ "HTTP/1.0 200 OK\r\n"};
///HTTP Header: 400 Bad Request
PROGMEM char http_header_400[]={ "HTTP/1.0 400 Bad Request\r\n"};
///HTTP Header: 401 Unauthorized
PROGMEM char http_header_401[]={ "HTTP/1.0 401 Unauthorized\r\n"};
///HTTP Header parts: server
PROGMEM char http_header_404[]={ "HTTP/1.0 404 Not Found\r\n"};
///HTTP Header parts: server
PROGMEM char head_http_server[]= { "Server: AVR_Small_Webserver\r\n"};
///HTTP Header parts: auth. required
PROGMEM char head_authenticate[]={ "WWW-Authenticate: Basic realm=\"NeedPassword\"\r\n"};
///HTTP Header parts: content type
PROGMEM char head_content_type[]={ "Content-Type: "};
///HTTP Header parts: content type default
PROGMEM char head_content_type_default[]={ "text/html"};
///HTTP Header parts: header_end
PROGMEM char head_end[] = {"\r\n\r\n"};

///HTTP Header buffer: encoded password given from config.h
char http_auth_passwort[30];


//----------------------------------------------------------------------------

///Simple file: Empty text
PROGMEM char Page_Empty[] = {""};
///Simple file: Bad request text
PROGMEM char Page_400[] = {"400 Bad Request"};
///Simple file: Unauthorized text
PROGMEM char Page_401[] = {"401 Unauthorized"};
///Simple file: Not found text
PROGMEM char Page_404[] = {"404 Not Found"};

///Requests a file type for a given filename
char(*file_type)(char *, char *);
///Executes a given variable with given content
char(*exec_var)(char, char *,char *);
///Reads a data segment from a given filename
unsigned int (*file_read)(char *buf,char *filename,unsigned int start,unsigned int maxlen);

//----------------------------------------------------------------------------

/**Initialize the HTTP daemon

	\param fp1 callback for evaluating the filetype transmitted in the HTTP header
	\param fp2 callback for evaluating variables in HTTP POST packages
	\param fp3 callback for reading a file segment of a requested file sent via HTTP
*/
void httpd_init (char(*fp1)(char *, char *),char(*fp2)(char, char *,char *),unsigned int(*fp3)(char *,char *,unsigned int,unsigned int))
{
	//HTTP_AUTH_STRING 
	decode_base64((unsigned char*)HTTP_AUTH_STRING,(unsigned char *)http_auth_passwort);

	//Serverport und Anwendung eintragen
	add_tcp_app (HTTPD_PORT, (void(*)(unsigned char))httpd);

	file_type = *fp1;
	exec_var = *fp2;
	file_read = *fp3;
}
   
//----------------------------------------------------------------------------

/**Callback function for HTTP daemon (called when a TCP package for the HTTP
   port is captured).

	\param index handle to item on TCP stack
*/
void httpd (unsigned char index)
{
    //Verbindung wurde abgebaut!
    if (tcp_entry[index].status & FIN_FLAG)
    {
        return;
    }

	//Allererste Aufruf des Ports fr diese Anwendung
	//HTTPD_Anwendungsstack lschen
	if(tcp_entry[index].app_status==1)
	{
		httpd_stack_clear(index);
	}
	
	//HTTP wurde bei dieser Verbindung zum ersten mal aufgerufen oder
	//HTML Header Retransmission!
	if (tcp_entry[index].app_status <= 2)
	{	
		httpd_header_check (index);
		return;
	}

	//Der Header wurde gesendet und mit ACK besttigt (tcp_entry[index].app_status+1)
	//war das HTML Paket fertig, oder mssen weitere Daten gesendet werden, oder Retransmission?
	if (tcp_entry[index].app_status > 2 && tcp_entry[index].app_status < 0xFFFE)
	{
		httpd_data_send (index);
		return;
	}
	
	//Verbindung kann geschlossen werden! Alle HTML Daten wurden gesendet TCP Port kann
	//geschlossen werden (tcp_entry[index].app_status >= 0xFFFE)!!
	if (tcp_entry[index].app_status >= 0xFFFE)
	{
		tcp_entry[index].app_status = 0xFFFE;
		tcp_Port_close(index);
		return;
	}
}

//----------------------------------------------------------------------------

/**Clears the HTTP table entry. Called on begin of a HTTP communication.

	\param index handle to item on HTTP stack (lies in parallel to TCP stack)
*/
void httpd_stack_clear (unsigned char index)
{
	http_entry[index].http_auth = HTTP_AUTH_DEFAULT;
	http_entry[index].new_page_pointer = Page_Empty;
	http_entry[index].old_page_pointer = Page_Empty;
	http_entry[index].new_filepos = 0;
	http_entry[index].old_filepos = 0;
	http_entry[index].first = 1;
	http_entry[index].send_file = 0;
	http_entry[index].post = 0;
	http_entry[index].post_name = 1;
	http_entry[index].post_val = 0;
	http_entry[index].var_cnt = 0;
	http_entry[index].auth_ptr = http_auth_passwort;
	http_entry[index].hdr_end_pointer = head_end;
	strcpy_P(http_entry[index].file_name,PSTR("\0"));
	strcpy_P(http_entry[index].file_type,head_content_type_default);

	HTTP_DEBUGP("\n*** NEUE HTTP ANFORDERUNG ***\n");	

	return;
}

//----------------------------------------------------------------------------
/**Checks the transmitted HTTP header. Performs check of request type, filename
   and proper authorization (if needed). Also prepares the HTTP reply header.

	\param index handle to item on HTTP stack (lies in parallel to TCP stack)
*/
void httpd_header_check (unsigned char index)
{
	unsigned int a;
	//HTTP_DEBUGP("CHECK HEADER #%x\n",index);

#if HTTP_DEBUG	
	void *x;
	for (a=1024;a>0;a-=1) {		
		x=malloc(a);
		if (x) {
			free(x);
			HTTP_DEBUGP("At least %d bytes free\n",a);
			break;
		}
	}
#endif
	
	//GET <URL> wird ganz am Anfang, noch vor einer mglichen 
	//Authentifizierungs-Sequenz gesendet!
	if 	(http_entry[index].first &&
	     (strncasecmp_P((char*)&(eth_buffer[TCP_DATA_START_VAR]),PSTR("GET "),4)==0)) 
	{
		HTTP_DEBUGP("GET PACKAGE\n");

		//Wir hngen eine String-Endeerkennung nach dem Filenamen an und
		//kopieren den Namen fr spter.
		for(a = TCP_DATA_START_VAR+4;a<TCP_DATA_END_VAR;a++)
		{
			//HTTP_DEBUGP("%c",eth_buffer[a]<0x20?0x80:eth_buffer[a]);
			if ((eth_buffer[a] == '\r') || (eth_buffer[a] == ' '))
			{
				eth_buffer[a] = '\0';
				break;
			}
		}
		strncpy(http_entry[index].file_name,(char *)&(eth_buffer[TCP_DATA_START_VAR+5]),MAX_FILENAME_SIZE-1);
		HTTP_DEBUGP("URL: '%s'\n",http_entry[index].file_name);

		//Signalisieren, dass wir einen Filenamen zum Senden haben!
		http_entry[index].send_file=1;
	}

	//POST <URL> wird ganz am Anfang, noch vor einer mglichen 
	//Authentifizierungs-Sequenz gesendet!
	if 	(http_entry[index].first &&
	     (strncasecmp_P((char*)&(eth_buffer[TCP_DATA_START_VAR]),PSTR("POST "),5)==0))
	{
		HTTP_DEBUGP("POST PACKAGE\n");

		//Wir hngen eine String-Endeerkennung nach dem Filenamen an und
		//kopieren den Namen fr spter.
		for(a = TCP_DATA_START_VAR+5;a<(TCP_DATA_END_VAR);a++)
		{
			//HTTP_DEBUGP("%c",eth_buffer[a]<0x20?0x80:eth_buffer[a]);
			if ((eth_buffer[a] == '\r') || (eth_buffer[a] == ' '))
			{
				eth_buffer[a] = '\0';
				break;
			}
		}
		strncpy(http_entry[index].file_name,(char *)&(eth_buffer[TCP_DATA_START_VAR+5]),MAX_FILENAME_SIZE-1);
		HTTP_DEBUGP("URL: '%s'\n",http_entry[index].file_name);

		//Signalisieren, dass wir einen Filenamen zum Senden haben!
		http_entry[index].send_file=1;
		//Signalisieren, dass Daten zur weiteren Verarbeitung folgen!
		http_entry[index].post = 1;
	}
	
	//Weitere Pakete haben keinen Einflu mehr auf die obige HTTP-Methodenerkennung!
	http_entry[index].first=0;

	//Wir lesen den Inhalt von Beginn an, ggf. mssen wir zuerst das Headerende finden
	//(nchste Schleife) gefolgt von einer mglichen POST-Daten Leseschleife.
	a=TCP_DATA_START_VAR;

	//finden der Authorization und das Ende im Header auch ber mehrere Pakete hinweg!!	
	if(pgm_read_byte(http_entry[index].hdr_end_pointer))
	{		
		HTTP_DEBUGP("FIND AUTH...");
		for(;a<(TCP_DATA_END_VAR);a++)
		{	
			//HTTP_DEBUGP("%c",eth_buffer[a]);
			if(!http_entry[index].http_auth) 
			{
				if (eth_buffer[a] != *http_entry[index].auth_ptr++)
				{
					http_entry[index].auth_ptr = http_auth_passwort;
				}
				if(*http_entry[index].auth_ptr == 0) 
				{
					http_entry[index].http_auth = 1;
					HTTP_DEBUGP("...AUTH OK!\n");
				}
			}
			
			//Das Headerende wird mit (CR+LF+CR+LF) angezeigt!
			if (eth_buffer[a] != pgm_read_byte(http_entry[index].hdr_end_pointer++))
			{
				http_entry[index].hdr_end_pointer = head_end;
			}
			if (!pgm_read_byte(http_entry[index].hdr_end_pointer)) 
			{
				HTTP_DEBUGP("...AUTH DONE\n");
				break;
			}
		}
	}
	
	//NUR wenn wir authorisiert wurden, lesen wir ggf. ein POST Paket ein!
	//Hier extrahieren wir sofort Variablennamen und deren Inhalte.
	if ( (http_entry[index].http_auth && http_entry[index].post==1) &&
	     (!pgm_read_byte(http_entry[index].hdr_end_pointer))          )
	{
		HTTP_DEBUGP("POST PACKET...\n");
		for(;a<(TCP_DATA_END_VAR);a++)
		{	char c = eth_buffer[a];
		
			//HTTP_DEBUGP("%c",c<0x20?0x80:c);

			//Wir lesen den Name/Valuestring ein
			if ((http_entry[index].post_name) && (http_entry[index].var_cnt<MAX_NAME_SIZE-1)) {
				if ((c!='=') && (c>' ')) http_entry[index].var_name[http_entry[index].var_cnt++]=c;
			}			
			if ((http_entry[index].post_val) && (http_entry[index].var_cnt<MAX_VAL_SIZE-1)) {
				if ((c!='&') && (c>' ')) http_entry[index].var_value[http_entry[index].var_cnt++]=c;
			}

			//Ggf. schalten wir von Namestring auf Valuestring um
			if ((http_entry[index].post_name) && (c=='=')) {
				http_entry[index].var_name[http_entry[index].var_cnt]=0;
				http_entry[index].var_cnt=0;
				http_entry[index].post_name=0;
				http_entry[index].post_val=1;
				http_entry[index].var_value[http_entry[index].var_cnt]=0;
			} 

			//Die Variablenexekution liefert eine Null, um das Einlesen 
			//der POST-Daten zu beenden. Eine Null bei der bergabe signalisiert
			//der Exekutionsroutine, dass die Daten vollstndig sind.
			if ( (http_entry[index].post_val) && (c=='&') ) {
				http_entry[index].var_value[http_entry[index].var_cnt]=0;
				http_entry[index].var_cnt=0;
				HTTP_DEBUGP("#%s#%s#\n",http_entry[index].var_name,http_entry[index].var_value);
				//HOOK: Analyze post data.
				if (!exec_var(0,http_entry[index].var_name,http_entry[index].var_value)) {
					http_entry[index].post = 0;
					HTTP_DEBUGP("...POST DONE\n");
					break;
				}
				//HOOK END: Analyze post data.				
				http_entry[index].post_name=1;
				http_entry[index].post_val=0;
				http_entry[index].var_name[http_entry[index].var_cnt]=0;
			}
			//Am Ende einer Value-Deklaration in der POST-bertragung folgt 
			//kein '&'. Somit wissen wir am Paketende nicht, ob die Daten
			//vollstndig sind. Daher signalisieren wir der Variablenexekutions-
			//routine diesen Sachverhalt mit dem Parameter Incomplete=1.
			//Soferne der Name nicht das POST-Ende signalisiert, kann die
			//Exekutionsroutine die Verarbeitung unterdrcken und auf ein
			//nchstes Paket warten. Wir schalten daher noch nicht auf Namensstring!
			if ( (http_entry[index].post_val) && (a==TCP_DATA_END_VAR-1) ) {
				http_entry[index].var_value[http_entry[index].var_cnt]=0;
				HTTP_DEBUGP("#%s#%s#\n",http_entry[index].var_name,http_entry[index].var_value);
				//HOOK: Analyze post data.
				if (!exec_var(1,http_entry[index].var_name,http_entry[index].var_value)) {
					http_entry[index].post = 0;
					HTTP_DEBUGP("...POST DONE\n");
					break;
				}
				//HOOK END: Analyze post data.				
			}
		}
	}	
	
	//Wurde das Ende vom Header nicht erreicht
	//kommen noch weitere Stcke vom Header!
	if ( (http_entry[index].post==1) ||
         (pgm_read_byte(http_entry[index].hdr_end_pointer)!=0) ) 
	{
		HTTP_DEBUGP("CONTINUE READ\n");
		//Der Empfang wird quitiert und es wird auf weiteres Headerstck gewartet
		tcp_entry[index].status =  ACK_FLAG;
		create_new_tcp_packet(0,index);
		//Warten auf weitere Headerpakete
		tcp_entry[index].app_status = 1;
		return;
	}	
	
	//Wir sind durch mit der Anfrage, nun bereiten wir die Antwort vor!
	tcp_entry[index].app_status = 2;

	//Zuerst prfen wir, ob wir eine (fr uns) korrekte Anfrage bekommen haben.

	//Wurde das Passwort in den ganzen Headerpaketen gefunden?
	//Wenn nicht dann das Passwort anfordern!
	if((!http_entry[index].http_auth) && tcp_entry[index].status&PSH_FLAG)
	{	HTTP_DEBUGP("REQUEST AUTH!\n");

		//Zugehrigen Inhalt festlegen.
		http_entry[index].new_page_pointer = Page_401;
		strcpy_P(http_entry[index].file_name,PSTR("Page_401\0"));

		//Und raus mit dem Header!
		httpd_header_send(index,http_header_401);
		return;
	}

	//Passwort (ggf.) ok, aber keine brauchbare GET/POST Anforderung empfangen!
	if (!http_entry[index].send_file) 
	{	HTTP_DEBUGP("BAD REQUEST!\n");

		//Zugehrigen Inhalt festlegen.
		http_entry[index].new_page_pointer = Page_400;
		strcpy_P(http_entry[index].file_name,PSTR("Page_400\0"));

		//Und raus mit dem Header!
		httpd_header_send(index,http_header_400);
		return;
	}
		
	//Die Anfrage passt soweit, jetzt checken wir den Filetyp und richten
	//einen entsprechenden Header her, wenn wir nichts gefunden haben.
		
	//HOOK: check file and setup type!
	if (!file_type(http_entry[index].file_name,http_entry[index].file_type)) 
	{	HTTP_DEBUGP("FILE NOT FOUND!\n");
		
		//Zugehrigen Inhalt festlegen.
		http_entry[index].new_page_pointer = Page_404;
		strcpy_P(http_entry[index].file_name,PSTR("Page_404\0"));

		//Und raus mit dem Header!
		httpd_header_send(index,http_header_404);
		return;
	}
	//HOOK END: check file and setup type!

	HTTP_DEBUGP("SEND FILE %s\n",http_entry[index].file_type);
	httpd_header_send(index,http_header_200);
}

//----------------------------------------------------------------------------

/**Sends the HTTP header

	\param index handle to item on HTTP stack (lies in parallel to TCP stack)
	\param ptr message header to send (PSTR)
*/
void httpd_header_send (unsigned char index, PGM_P ptr)
{	
	unsigned int a;
	HTTP_DEBUGP("SEND HEADER\n");

	//Header zusammenbauen.
	a=0;
	strcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR+a],ptr);
	a+=strlen_P(ptr);
	memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR+a],head_http_server,sizeof(head_http_server)-1);
	a+=sizeof(head_http_server)-1;
	memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR+a],head_authenticate,sizeof(head_authenticate));
	a+=sizeof(head_authenticate)-1;
	memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR+a],head_content_type,sizeof(head_content_type));
	a+=sizeof(head_content_type)-1;
	memcpy((char*)&eth_buffer[TCP_DATA_START_VAR+a],http_entry[index].file_type,sizeof(http_entry[index].file_type));
	a+=strlen(http_entry[index].file_type);
	memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR+a],head_end,sizeof(head_end));
	a+=sizeof(head_end)-1;
	
	//Und raus mit dem Paket!
	tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
	create_new_tcp_packet(a,index);
}

//----------------------------------------------------------------------------

/**Sends the requested HTTP content. Might be a given file retreived by 
   the callback functions or just sends some short, predefined answers.

	\param index handle to item on HTTP stack (lies in parallel to TCP stack)
*/
void httpd_data_send (unsigned char index)
{	
	unsigned int a;

	HTTP_DEBUGP("SEND DATA...\n");

	//Passwort wurde im Header nicht gefunden
	if(!http_entry[index].http_auth)
	{
		HTTP_DEBUGP("BLOCKED\n");
		//Zugehrigen Inhalt festlegen.
		http_entry[index].new_page_pointer = Page_401;
		strcpy_P(http_entry[index].file_name,PSTR("Page_401\0"));
	}

	//kein Paket empfangen Retransmission des alten Paketes
	if (tcp_entry[index].status == 0) 
	{
		HTTP_DEBUGP("RESEND\n");
		http_entry[index].new_filepos = http_entry[index].old_filepos;		
		http_entry[index].new_page_pointer = http_entry[index].old_page_pointer;
	}
	http_entry[index].old_filepos = http_entry[index].new_filepos;		
	http_entry[index].old_page_pointer = http_entry[index].new_page_pointer;

	//Try to read from the given filename
	//HOOK: Read file segment (if any)
	a=file_read((char *)&(eth_buffer[TCP_DATA_START]),http_entry[index].file_name,
	            http_entry[index].new_filepos,MTU_SIZE-(TCP_DATA_START)-150);
	//HOOK END: Read file segment (if any)

	//We are at the beginning of a file and the file read returns zero...
	if (!(http_entry[index].new_filepos || a)) {
		//...then we could not get a file, so we use some predefined file contents
		HTTP_DEBUGP("    PREDEFINED\n");
		for (a = 0;a<(MTU_SIZE-(TCP_DATA_START)-150);a++)
		{
			unsigned char b;
			b = pgm_read_byte(http_entry[index].new_page_pointer++);
			eth_buffer[TCP_DATA_START + a] = b;
			
			//wurde das Ende des Paketes erreicht?
			//Verbindung TCP Port kann beim nchsten ACK geschlossen werden
			//Schleife wird abgebrochen - keine Daten mehr!!
			if (!b)
			{
				tcp_entry[index].app_status = 0xFFFD;
				break;
			}
		}
	} else {
		//We got a file and send the segment, so we increment the file pointer accordingly.
		HTTP_DEBUGP("    DYNAMIC %x from %x\n",a,http_entry[index].new_filepos);
		http_entry[index].new_filepos+=a;
		//Buffer filled incompletely, assuming we are ready.
		if (a != (MTU_SIZE-(TCP_DATA_START)-150))
		{	
			HTTP_DEBUGP("...DATA END\n");
			tcp_entry[index].app_status = 0xFFFD;
		}
	}
	//Erzeugte Paket kann nun gesendet werden!
	tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
	create_new_tcp_packet(a,index); //Sends package and increments app_status
	return;
}

#endif //USE_HTTPD
